/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _SPO_MONITOR_H_
#define _SPO_MONITOR_H_

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <openssl/ssl.h>
#include <openssl/x509.h>
#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>

#define errorMessage(arg...)                        \
  fprintf(stderr, "%s:%d ", __FILE__, __LINE__),    \
  fprintf(stderr, ##arg)

#ifdef DEBUG
#define debugMessage(arg...)                        \
  fprintf(stderr, ##arg)
#else
#define debugMessage(arg...)
#endif

/* Get the first in the pool */
#define GETFIRST(p, pf) \
  for(pf=p; pf->pPrev; pf=pf->pPrev)

/* Get the last */
#define GETLAST(p, pl) \
  for(pl=p; pl->pNext; pl=pl->pNext)

typedef struct confgInfo
{
  char mntrIP[16];
  unsigned short usPort;
  unsigned long ulSid;
  char *pCertificate;
} CONFGINFO;

typedef struct sensorEvent 
{
  unsigned long ulMsgId;
  unsigned long ulSensorId;
  unsigned long ulSgntrId;
  struct timeval eventTime;
  unsigned char ucSrcMac[6];
  unsigned char ucDstMac[6];
  unsigned short usPrtclID;
  unsigned short padToLong;
  unsigned long ulSrcIp;
  unsigned long ulDstIp;
  unsigned short usSrcPort;
  unsigned short usDstPort;
  unsigned short usPriority;
  unsigned short usEvtLen;
  char *pEventMsg;
} SNSREVENT;

typedef struct sensorStatistic
{
  unsigned long ulMsgId;
  unsigned long ulSensorId;
  struct timeval sensorTime;
  unsigned long ulAltPkts;
  unsigned long ulLogPkts;
  unsigned long ulPsdPkts;
  unsigned long ulTcpPkts;
  unsigned long ulUdpPkts;
  unsigned long ulIcmpPkts;
  unsigned long ulArpPkts;
  unsigned long ulIpxPkts;
  unsigned long ulOthPkts;
  unsigned long ulDscdPkts;
  unsigned long ulFrgIPPkts;
  unsigned long ulRbtIpPkts;
  unsigned long ulStrmTcpPkts;
  unsigned long ulRcnsTcpPkts;
  unsigned long ulRcnsStrmPkts;
} SNSRSTATS;

typedef struct eventPool
{
  SNSREVENT *pEvent;
  struct eventPool *pPrev;
  struct eventPool *pNext;
} EVENTPOOL;

typedef struct snsrMonitor
{
  CONFGINFO *pCfg;
  SNSRSTATS *pStats;
  EVENTPOOL *pEvtPl;
  pthread_t tMainId;
  pthread_t tSpndId;
  pthread_mutex_t tMutex;
  pthread_cond_t tCond;
  int iSockFd;
  char cException;
  unsigned char ucCtrlMsg;
} SNSRMONIT;

#define CHAINDEPTH  1
#define EVENTMSG  0x1a2b3c40
#define STATIMSG  0x1a2b3c41
#define PSEUDOHDLEN  (sizeof(SNSREVENT) - 4)
#define STATSTUBLEN  (sizeof(SNSRSTATS) - PSEUDOHDLEN)

#define SENSORUP  0x1
#define SNSRSUSP  0x2
#define SNSRSTOP  0x4

#define MSGTIMEINTV  20

void SetupMonitor();

#endif
