/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * $Id$
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>

#include <printmsg.h>
#include <gmlmonit.h>
#include <snortdata.h>
#include <sqlodbc.h>
#include <dbprocs.h>

void *doDbInsertThread(void *pUser)
{
  MONITSRVR *pMs = pUser;
  SNSREVENT *pEvt = NULL;
  struct timeval tTmStmp = {0, 0};

  /* ODBD stuff */
  void *pHEnv = NULL;
  void *pHDbc = NULL;
  void *pHStm = NULL;

  SQLMSG tSqlMsg;
  unsigned long ulAddrId = 0;
  unsigned long ulEvntId = 0;
  char stmt[128];
  short sNumPm = 0;
  short sClms = 0;
  int iRows = 0;
  int iDbErr = 0;
  char *pDbName = NULL;
  char *pDbUser = NULL;
  char *pPsWord = NULL;

  if(!pMs)
    return NULL;

  debugMessage("Enter DbInsert thread: %ld\n", gmlThreadId());

  if(!(pDbName = getCnfgCharElement("dbname", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase name failed ...\n");
    pMs->uiNoDbs = 1;
  }

  if(!pMs->uiNoDbs)
  {
    if(!(pDbUser = getCnfgCharElement("dbuser", pMs->pSrvrCnfg)))
      errorMessage("Get DataBase user name failed ...\n");

    if(!(pPsWord = getCnfgCharElement("psword", pMs->pSrvrCnfg)))
      errorMessage("Get DataBase user password failed ...\n");

    if(dbInitEnv(&pHEnv, &tSqlMsg))
    {
      errorMessage("dbInitDbExit failed ... %s %ld\n",
                                  tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
      pMs->uiNoDbs = 1;
    }
    else if(dbConnect(pDbName, pDbUser, pPsWord, pHEnv, &pHDbc, 0, &tSqlMsg))
    {
      errorMessage("dbConnect failed ... %s %ld\n",
                                  tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
      pMs->uiNoDbs = 1;
    }
  }

  if(!pMs->uiNoDbs)
  {
    sprintf(stmt, "select max(addrId) from addrinfo");

    if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iDbErr = 1;
    }
    else
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iDbErr = 1;
      }
      else if(iRows == 1 && sClms == 1)
      {
        if(sqlGetOne(pHEnv, pHDbc, pHStm, SQL_INTEGER,
                                          &ulAddrId, 4, 0, &tSqlMsg))
        {
          errorMessage("sqlGetOen failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iDbErr = 1;
        }
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, 0, NULL, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iDbErr = 1;
    }
  }

  if(!pMs->uiNoDbs)
  {
    memset(stmt, 0, sizeof(stmt));
    sprintf(stmt, "select max(eventId) from events");

    if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iDbErr = 1;
    }
    else
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iDbErr = 1;
      }
      else if(iRows == 1 && sClms == 1)
      {
        if(sqlGetOne(pHEnv, pHDbc, pHStm, SQL_INTEGER,
                                          &ulEvntId, 4, 0, &tSqlMsg))
        {
          errorMessage("sqlGetOen failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iDbErr = 1;
        }
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, 0, NULL, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iDbErr = 1;
    }
  }

  while(!pMs->cException)
  {
    GDATAPOLL *pFirst = NULL;

    gmlMxLock(pMs->pDtMtx);

    GETFIRST(pMs->pClntData, pFirst);

#define CLNTDATA pFirst->pNext->pAny
    if(!pFirst->pNext || !CLNTDATA)
      gmlCdTimedWait(pMs->pCnd, pMs->pDtMtx, 4000);
    else
    {
      unsigned long ulMsgID = *(unsigned long *)CLNTDATA;

      while(!pMs->uiNoDbs && !iDbErr)
      {
        if(ntohl(ulMsgID) == EVENTMSG)
        {
          pEvt = (SNSREVENT *)CLNTDATA;
#undef CLNTDATA

          if(timeValComp(&pFirst->pNext->tStamp, &tTmStmp)>0)
          {
            unsigned long ulAddrTmp = 0;
            char *pEvtStmt = NULL;
            int iRc = 0;

            if(dirExec(pHEnv, pHDbc, "begin work", &tSqlMsg))
            {
              errorMessage("dirExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
              break;
            }

            memset(stmt, 0, sizeof(stmt));
            sprintf(stmt, "insert into addrinfo values (%lu, %lu, %lu)",
                        ++ulAddrId, (unsigned long)ntohl(pEvt->ulSrcIp),
                        (unsigned long)ntohl(pEvt->ulDstIp));

            if(dirExec(pHEnv, pHDbc, stmt, &tSqlMsg))
            {
              --ulAddrId;
              memset(stmt, 0, sizeof(stmt));
              sprintf(stmt,
                  "select addrId from addrinfo where srcIp=%lu and dstIp=%lu",
                  (unsigned long)ntohl(pEvt->ulSrcIp),
                  (unsigned long)ntohl(pEvt->ulDstIp));
                  
              if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm, &sNumPm, &tSqlMsg))
              {
                errorMessage("sqlPrep failed ... %ld %s\n",
                                         tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
                iDbErr = 1;
              }
              else
              {
                if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
                {
                  errorMessage("sqlExec failed ... %ld %s\n",
                                           tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
                  iDbErr = 1;
                }
                else if(iRows == 1 && sClms == 1)
                {
                  if(sqlGetOne(pHEnv, pHDbc, pHStm, SQL_INTEGER,
                                              &ulAddrTmp, 4, 0, &tSqlMsg))
                  {
                    errorMessage("sqlGetOen failed ... %ld %s\n",
                                           tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
                    iDbErr = 1;
                  }
                }
              }

              if(sqlFree(pHEnv, pHDbc, pHStm, 0, NULL, &tSqlMsg))
              {
                errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
                iDbErr = 1;
              }
            }

            if(pMs->cException || iDbErr)
              break;
         
            if(!(pEvtStmt = (char *)calloc(1, 172 + strlen(pEvt->pEventMsg))))
            {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
              errorMessage("%s\n", strerror(errno));
#else
              errorMessage("calloc failed.\n");
#endif
              pMs->cException = 1;
              break;
            }

            sprintf(pEvtStmt, "insert into events values (%lu, %lu, %lu, %lu,"
                  "'%s', %d, %d, '%02X:%02X:%02X:%02X:%02X:%02X',"
                  "'%02X:%02X:%02X:%02X:%02X:%02X', %d, %d, %lu)",
                  ++ulEvntId,
                  (unsigned long)ntohl(pEvt->ulSensorId),
                  (unsigned long)ntohl(pEvt->ulSgntrId),
                  pEvt->eventTime.tv_sec, pEvt->pEventMsg,
                  ntohs(pEvt->usSrcPort), ntohs(pEvt->usDstPort),
                  pEvt->ucSrcMac[0], pEvt->ucSrcMac[1],
                  pEvt->ucSrcMac[2], pEvt->ucSrcMac[3],
                  pEvt->ucSrcMac[4], pEvt->ucSrcMac[5],
                  pEvt->ucDstMac[0], pEvt->ucDstMac[1],
                  pEvt->ucDstMac[2], pEvt->ucDstMac[3],
                  pEvt->ucDstMac[4], pEvt->ucDstMac[5],
                  pEvt->usPrtclID,
                  ntohs(pEvt->usPriority),
                  ulAddrTmp? ulAddrTmp : ulAddrId);

            iRc = dirExec(pHEnv, pHDbc, pEvtStmt, &tSqlMsg);

            free(pEvtStmt);

            if(iRc)
            {
              errorMessage("dirExec failed ... %ld %s\n",
                                       tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
              if(!ulAddrTmp)
                --ulAddrId;
              --ulEvntId;

              if(dirExec(pHEnv, pHDbc, "rollback", &tSqlMsg))
                errorMessage("dirExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);

              iDbErr = 1;

            }
            else
            {
              if(dirExec(pHEnv, pHDbc, "commit", &tSqlMsg))
              {
                errorMessage("dirExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
                iDbErr = 1;
              }
            }

            if(pMs->cException || iDbErr)
              break;
          }

          tTmStmp.tv_sec = pFirst->pNext->tStamp.tv_sec;
          tTmStmp.tv_usec = pFirst->pNext->tStamp.tv_usec;
        } /* if */

        iDbErr = 0;
        break;
      } /* while */

      if(pFirst->pNext->uiRfrncd == (unsigned int)pMs->ucActvClnts)
        deleteElement(pFirst->pNext);
    }

    gmlMxUnLock(pMs->pDtMtx);
  } /* while */

  if(dbDisconn(pHEnv, pHDbc, &tSqlMsg))
    errorMessage("dbDisconn failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);

  if(dbExitEnv(pHEnv, &tSqlMsg))
    errorMessage("dbEixtEnv failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);

  pMs->tDbiThrdId = 0;
  debugMessage("Leaving DbInsert thread ...\n");
  return NULL;
}

int dbFetchAtckInfo(void *pUser, const char *pStmt,
                    pFuncProc procData, const void *pObj)
{
  MONITSRVR *pMs = pUser;
  int iErr = 0;

  /* ODBD stuff */
  void *pHEnv = NULL;
  void *pHDbc = NULL;
  void *pHStm = NULL;
  ODBCBind *pDsc = NULL;
  ODBCBind *pTmpDsc = NULL;

  SQLMSG tSqlMsg;
  char stmt[512];
  short sNumPm = 0;
  short sClms = 0;
  int iRows = 0;
  unsigned long ulEvtTtl = 0;
  char *pDbName = NULL;
  char *pDbUser = NULL;
  char *pPsWord = NULL;

  if(!pStmt)
    return -1;

  debugMessage("In dbFetchAtckInfo ...\n");

  if(!(pDbName = getCnfgCharElement("dbname", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase name failed ...\n");
    return -1;
  }

  if(!(pDbUser = getCnfgCharElement("dbuser", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user name failed ...\n");
    return -1;
  }

  if(!(pPsWord = getCnfgCharElement("psword", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user password failed ...\n");
    return -1;
  }

  if(dbInitEnv(&pHEnv, &tSqlMsg))
  {
    errorMessage("dbInitDbExit failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
    iErr = 1;
  }
  else if(dbConnect(pDbName, pDbUser, pPsWord, pHEnv, &pHDbc, 0, &tSqlMsg))
  {
    errorMessage("dbConnect failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
    iErr = 1;
  }

  if(!iErr)
  {
    short sCndLen = (short)(strstr(pStmt, "group") - strstr(pStmt, "from"));

    memset(stmt, 0, sizeof(stmt));
    strcpy(stmt, "select count(*) ");
    strncat(stmt, strstr(pStmt, "from"), sCndLen);

    if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }
    else
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }
      else if(iRows == 1 && sClms == 1)
      {
        if(sqlGetOne(pHEnv, pHDbc, pHStm, SQL_INTEGER,
                                         &ulEvtTtl, 4, 0, &tSqlMsg))
        {
          errorMessage("sqlGetOen failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iErr = 1;
        }
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, 0, NULL, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }
  }

  if(!iErr)
  {
    void *pHStm1 = NULL;
    void *pHStm2 = NULL;
    short sCndLen = 0;
    short sNoBind = 0;

    memset(stmt, 0, sizeof(stmt));

    if(strstr(pStmt, "where"))
    {
      sCndLen = (short)(strstr(pStmt, "group") - strstr(pStmt, "where"));
      strcpy(stmt, "select count(distinct srcIp) from events a, "
                   "addrinfo b where b.addrId = a.addrId and");

      if(!strstr(pStmt, "signatureId ="))
        strcat(stmt, " signatureId = ? and");
      else
        sNoBind = 1;

      strncat(stmt, strstr(pStmt, "where")+5, sCndLen-5);
    }
    else
      strcpy(stmt, "select count(distinct srcIp) from events a, addrinfo b "
                   "where b.addrId = a.addrId and signatureId = ?");

    if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm1, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }

    if(!iErr)
    {
      memset(stmt, 0, sizeof(stmt));

      if(strstr(pStmt, "where"))
      {
        sCndLen = (short)(strstr(pStmt, "group") - strstr(pStmt, "where"));
        strcpy(stmt, "select count(distinct dstIp) from events a, "
                     "addrinfo b where b.addrId = a.addrId and");

        if(!strstr(pStmt, "signatureId ="))
          strcat(stmt, " signatureId = ? and");

        strncat(stmt, strstr(pStmt, "where")+5, sCndLen-5);
      }
      else
        strcpy(stmt, "select count(distinct dstIp) from events a, addrinfo b "
                     "where b.addrId = a.addrId and signatureId = ?");

      if(sqlPrep(pHEnv, pHDbc, stmt, &pHStm2, &sNumPm, &tSqlMsg))
      {
        errorMessage("sqlPrep failed ... %ld %s\n",
                                       tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }
    }

    if(!iErr)
    {
      if(sqlPrep(pHEnv, pHDbc, (char *)pStmt, &pHStm, &sNumPm, &tSqlMsg))
      {
        errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }
    }

    if(!iErr)
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }

      if(!iErr && sClms)
      {
        if(sqlDesc(pHEnv, pHDbc, pHStm, sClms, &pDsc, &tSqlMsg))
        {
          errorMessage("sqlDesc failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iErr = 1;
        }

        while(!sqlFetch(pHEnv, pHDbc, pHStm, &tSqlMsg) && !iErr)
        {
          ATCKINFO tAtckIf;
          unsigned short usMsgLen = 0;
          unsigned char *pDataBuff = NULL;
          char *pEvtMsg = NULL;
          short sTmpC = 0;
          int iTmpR = 0;
          int i = 0;

          memset(&tAtckIf, 0, ATCKIFLEN);

          for(i=0, pTmpDsc = pDsc; i < sClms; ++i, pTmpDsc++)
          {
            if(pTmpDsc->tIndicator != SQL_NULL_DATA)
            {
              switch(i)
              {
                case 0:
                        tAtckIf.usSigId = atoi((char *)pTmpDsc->pData);
                        break;
                case 1:
                        tAtckIf.sPrrty = atoi((char *)pTmpDsc->pData);
                        break;
                case 2:
                        tAtckIf.uiAtcks = atoi((char *)pTmpDsc->pData);
                        break;
                case 3:
                        pEvtMsg = (char *)pTmpDsc->pData;
                        usMsgLen = strlen((char *)pTmpDsc->pData);
                        tAtckIf.uiMsgLen =
                                 htonl(strlen((char *)pTmpDsc->pData));
                        break;
              }
            }
          }

          if(!sNoBind)
          {
            if(sqlBind(pHEnv, pHDbc, pHStm1, 1,
                            (char *)&tAtckIf.usSigId, C_SHORT, &tSqlMsg))
            {
              errorMessage("sqlBind failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
              break;
            }
          }

          if(sqlExec(pHEnv, pHDbc, pHStm1, &iTmpR, &sTmpC, &tSqlMsg))
          {
            errorMessage("sqlExec failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
            break;
          }

          if(sqlGetOne(pHEnv, pHDbc, pHStm1, SQL_INTEGER,
                                      &tAtckIf.uiAtckrs, 4, 1, &tSqlMsg))
          {
            errorMessage("sqlGetOen failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
            break;
          }

          if(!sNoBind)
          {
            if(sqlBind(pHEnv, pHDbc, pHStm2, 1,
                            (char *)&tAtckIf.usSigId, C_SHORT, &tSqlMsg))
            {
              errorMessage("sqlBind failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
              break;
            }
          }

          if(sqlExec(pHEnv, pHDbc, pHStm2, &iTmpR, &sTmpC, &tSqlMsg))
          {
            errorMessage("sqlExec failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
            break;
          }

          if(sqlGetOne(pHEnv, pHDbc, pHStm2, SQL_INTEGER,
                                        &tAtckIf.uiAtckd, 4, 1, &tSqlMsg))
          {
            errorMessage("sqlGetOen failed ... %ld %s\n",
                                          tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
            break;
          }

          tAtckIf.uiTtlRows = iRows; 
          tAtckIf.uiAtckt = ulEvtTtl;

          pDataBuff = (unsigned char *)calloc(1, ATCKIFLEN+usMsgLen+1);
  
          if(!pDataBuff)
          {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
            errorMessage("%s\n", strerror(errno));
#else
            errorMessage("calloc failed.\n");
#endif
            break;
          }

          memcpy(pDataBuff, &tAtckIf, ATCKIFLEN);
          memcpy(pDataBuff+ATCKIFLEN, pEvtMsg, usMsgLen);

          if(procData)
          {
            if((*procData)(pDataBuff, ATCKIFLEN+usMsgLen, pObj))
            {
              errorMessage("procData failed ...\n");
              break;
            }
          }
        } /* while */
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, sClms, pDsc, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }

    if(sqlFree(pHEnv, pHDbc, pHStm1, 0, NULL, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }

    if(sqlFree(pHEnv, pHDbc, pHStm2, 0, NULL, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }
  }

  if(dbDisconn(pHEnv, pHDbc, &tSqlMsg))
    errorMessage("dbDisconn failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);

  if(dbExitEnv(pHEnv, &tSqlMsg))
    errorMessage("dbEixtEnv failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
  return 0;
}

int dbFetchDetails(void *pUser, const char *pStmt,
                   pFuncProc procData, const void *pObj)
{
  MONITSRVR *pMs = pUser;
  int iErr = 0;

  /* ODBD stuff */
  void *pHEnv = NULL;
  void *pHDbc = NULL;
  void *pHStm = NULL;
  ODBCBind *pDsc = NULL;
  ODBCBind *pTmpDsc = NULL;

  SQLMSG tSqlMsg;
  short sNumPm = 0;
  short sClms = 0;
  int iRows = 0;
  char *pDbName = NULL;
  char *pDbUser = NULL;
  char *pPsWord = NULL;

  if(!pStmt)
    return -1;

  debugMessage("In dbFetchDetails ...\n");

  if(!(pDbName = getCnfgCharElement("dbname", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase name failed ...\n");
    return -1;
  }

  if(!(pDbUser = getCnfgCharElement("dbuser", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user name failed ...\n");
    return -1;
  }

  if(!(pPsWord = getCnfgCharElement("psword", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user password failed ...\n");
    return -1;
  }

  if(dbInitEnv(&pHEnv, &tSqlMsg))
  {
    errorMessage("dbInitDbExit failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
    iErr = 1;
  }
  else if(dbConnect(pDbName, pDbUser, pPsWord, pHEnv, &pHDbc, 0, &tSqlMsg))
  {
    errorMessage("dbConnect failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
    iErr = 1;
  }

  if(!iErr)
  {
    if(sqlPrep(pHEnv, pHDbc, (char *)pStmt, &pHStm, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }

    if(!iErr)
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }

      if(!iErr && sClms)
      {
        if(sqlDesc(pHEnv, pHDbc, pHStm, sClms, &pDsc, &tSqlMsg))
        {
          errorMessage("sqlDesc failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iErr = 1;
        }

        while(!sqlFetch(pHEnv, pHDbc, pHStm, &tSqlMsg) && !iErr)
        {
          DTLINFO tDtlIf;
          unsigned short usMsgLen = 0;
          unsigned char *pDataBuff = NULL;
          char *pEvtMsg = NULL;
          int i = 0;

          memset(&tDtlIf, 0, DTLIFLEN);

          for(i=0, pTmpDsc = pDsc; i < sClms; ++i, pTmpDsc++)
          {
            if(pTmpDsc->tIndicator != SQL_NULL_DATA)
            {
              switch(i)
              {
                case 0:
                        tDtlIf.uiSnsrId = atol((char *)pTmpDsc->pData);
                        break;
                case 1:
                        tDtlIf.ulEvtTime = strtoul((char *)pTmpDsc->pData,
                                                   NULL, 10);
                        break;
                case 2:
                        tDtlIf.usPrtclId = atoi((char *)pTmpDsc->pData);
                        break;
                case 3:
                        tDtlIf.sPrrty = atoi((char *)pTmpDsc->pData);
                        break;
                case 4:
                        tDtlIf.ulSrcIp = strtoul((char *)pTmpDsc->pData,
                                                 NULL, 10);
                        break;
                case 5:
                        tDtlIf.usSrcPt = atoi((char *)pTmpDsc->pData);
                        break;
                case 6:
                        tDtlIf.ulDstIp = strtoul((char *)pTmpDsc->pData,
                                                 NULL, 10);
                        break;
                case 7:
                        tDtlIf.usDstPt = atoi((char *)pTmpDsc->pData);
                        break;
                case 8:
                        strncpy(tDtlIf.cSrcMac, (char *)pTmpDsc->pData, 18);
                        break;
                case 9:
                        strncpy(tDtlIf.cDstMac, (char *)pTmpDsc->pData, 18);
                        break;
                case 10:
                        pEvtMsg = (char *)pTmpDsc->pData;
                        usMsgLen = strlen((char *)pTmpDsc->pData);
                        tDtlIf.uiMsgLen =
                                 htonl(strlen((char *)pTmpDsc->pData));
                        break;
              }
            }
          }

          tDtlIf.uiTtlRows = iRows; 

          pDataBuff = (unsigned char *)calloc(1, DTLIFLEN+usMsgLen+1);
  
          if(!pDataBuff)
          {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
            errorMessage("%s\n", strerror(errno));
#else
            errorMessage("calloc failed.\n");
#endif
            break;
          }

          memcpy(pDataBuff, &tDtlIf, DTLIFLEN);
          memcpy(pDataBuff+DTLIFLEN, pEvtMsg, usMsgLen);

          if(procData)
          {
            if((*procData)(pDataBuff, DTLIFLEN+usMsgLen, pObj))
            {
              errorMessage("procData failed ...\n");
              break;
            }
          }
        } /* while */
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, sClms, pDsc, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }
  }

  if(dbDisconn(pHEnv, pHDbc, &tSqlMsg))
    errorMessage("dbDisconn failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);

  if(dbExitEnv(pHEnv, &tSqlMsg))
    errorMessage("dbEixtEnv failed ... %s %ld\n",
                                tSqlMsg.pErrMsg, tSqlMsg.sqlcd);
  return 0;
}

int dbFetchReference(void *pUser, const char *pStmt,
                     pFuncProc procData, const void *pObj)
{
  MONITSRVR *pMs = pUser;
  int iErr = 0;

  /* ODBD stuff */
  void *pHEnv = NULL;
  void *pHDbc = NULL;
  void *pHStm = NULL;
  ODBCBind *pDsc = NULL;

  SQLMSG tSqlMsg;
  short sNumPm = 0;
  short sClms = 0;
  int iRows = 0;
  char *pDbName = NULL;
  char *pDbUser = NULL;
  char *pPsWord = NULL;

  if(!pStmt)
    return -1;

  debugMessage("In dbFetchReference...\n");

  if(!(pDbName = getCnfgCharElement("dbname", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase name failed ...\n");
    return -1;
  }

  if(!(pDbUser = getCnfgCharElement("dbuser", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user name failed ...\n");
    return -1;
  }

  if(!(pPsWord = getCnfgCharElement("psword", pMs->pSrvrCnfg)))
  {
    errorMessage("Get DataBase user password failed ...\n");
    return -1;
  }

  if(dbInitEnv(&pHEnv, &tSqlMsg))
  {
    errorMessage("dbInitDbExit failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
    iErr = 1;
  }
  else if(dbConnect(pDbName, pDbUser, pPsWord, pHEnv, &pHDbc, 0, &tSqlMsg))
  {
    errorMessage("dbConnect failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
    iErr = 1;
  }

  if(!iErr)
  {
    if(sqlPrep(pHEnv, pHDbc, (char *)pStmt, &pHStm, &sNumPm, &tSqlMsg))
    {
      errorMessage("sqlPrep failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }

    if(!iErr)
    {
      if(sqlExec(pHEnv, pHDbc, pHStm, &iRows, &sClms, &tSqlMsg))
      {
        errorMessage("sqlExec failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
        iErr = 1;
      }

      if(!iErr && iRows)
      {
        if(sqlDesc(pHEnv, pHDbc, pHStm, sClms, &pDsc, &tSqlMsg))
        {
          errorMessage("sqlDesc failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
          iErr = 1;
        }

        while(!sqlFetch(pHEnv, pHDbc, pHStm, &tSqlMsg) && !iErr)
        {
          unsigned short usMsgLen = 0;
          char *pDataBuff = NULL;

          usMsgLen = strlen((char *)pDsc->pData);

          pDataBuff = (char *)calloc(1, 1+usMsgLen+1);

          if(!pDataBuff)
          {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
            errorMessage("%s\n", strerror(errno));
#else
            errorMessage("calloc failed.\n");
#endif
            break;
          }

          *pDataBuff = (char)iRows;
          memcpy(pDataBuff+1, pDsc->pData, usMsgLen);

          if(procData)
          {
            if((*procData)(pDataBuff, usMsgLen+1, pObj))
            {
              errorMessage("procData failed ...\n");
              iErr = 1;
              free(pDataBuff);
              break;
            }
          }

          free(pDataBuff);
        } /* while */
      }
    }

    if(sqlFree(pHEnv, pHDbc, pHStm, sClms, pDsc, &tSqlMsg))
    {
      errorMessage("sqlFree failed ... %ld %s\n",
                                     tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      iErr = 1;
    }
  }

  if(dbDisconn(pHEnv, pHDbc, &tSqlMsg))
    errorMessage("dbDisconn failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);

  if(dbExitEnv(pHEnv, &tSqlMsg))
    errorMessage("dbEixtEnv failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
  return 0;
}
