/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <printmsg.h>
#include <gdatapool.h>

inline GDATAPOLL *newElement(GDATAPOLL *pElm)
{
   GDATAPOLL *pLast = NULL;

  if(!pElm)
    return NULL;

  GETLAST(pElm, pLast);

  /* Allocte a new one */
  if(!(pLast->pNext = (GDATAPOLL *)calloc(1, sizeof(GDATAPOLL))))
  {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
    errorMessage("%s\n", strerror(errno));
#else
    errorMessage("calloc failed.\n");
#endif

    return NULL;
  }

  pLast->pNext->pPrev = pLast;

  return pLast->pNext;
}

inline void deleteElement(GDATAPOLL *pElm)
{
  if(!pElm)
    return;

  if(pElm->pPrev)
    pElm->pPrev->pNext = pElm->pNext;

  if(pElm->pNext)
    pElm->pNext->pPrev = pElm->pPrev;

  if(pElm->pAny)
  {
    if(pElm->pfAnyFree)
      (*pElm->pfAnyFree)(pElm->pAny);

    free(pElm->pAny);
  }

  free(pElm);
}

inline void deleteAllElement(GDATAPOLL *pElm)
{
  GDATAPOLL *pFirst = NULL;

  if(!pElm)
    return;

  GETFIRST(pElm, pFirst);

  while(pFirst->pNext)
  {
    pFirst = pFirst->pNext;
    deleteElement(pFirst->pPrev);
  }

  deleteElement(pFirst);
}
