/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <netinet/in.h>

#include <printmsg.h>
#include <gmlmonit.h>
#include <gencerts.h>
#include <sslutils.h>
#include <snortdata.h>

/* Global data */
extern MONITSRVR tMonitSrv;

int initMonitServer(MONITSRVR *pMs)
{
  while(pMs)
  {
    if(!(pMs->pClntInfo = calloc(1, sizeof(GDATAPOLL))))
    {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
      errorMessage("%s\n", strerror(errno));
#else
      errorMessage("calloc failed.\n");
#endif

      break;
    }

    if(!(pMs->pClntData = calloc(1, sizeof(GDATAPOLL))))
    {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
      errorMessage("%s\n", strerror(errno));
#else
      errorMessage("calloc failed.\n");
#endif

      break;
    }

    if(!(pMs->pSrvrCnfg = calloc(1, sizeof(GDATAPOLL))))
    {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
      errorMessage("%s\n", strerror(errno));
#else
      errorMessage("calloc failed.\n");
#endif

      break;
    }

    if(!(pMs->pDtMtx = gmlMxInit()))
    {
      errorMessage("gmlMxInit failed.\n");
      break;
    }

    if(!(pMs->pIfMtx = gmlMxInit()))
    {
      errorMessage("gmlMxInit failed.\n");
      break;
    }

    if(!(pMs->pCnd = gmlCdInit()))
    {
      errorMessage("gmlCdInit failed.\n");
      break;
    }

    return 0;
  }

  dstryMonitServer(pMs);
  return -1;
}

void dstryMonitServer(MONITSRVR *pMs)
{
  if(pMs)
  {
    pMs->cException = 1;

#if 0
    /* Signal the thread */
    gmlCdSignal(pMs->pCnd);
#endif

    if(pMs->tExtThrdId)
      gmlThreadJoin(pMs->tExtThrdId);

    if(pMs->tDbiThrdId)
      gmlThreadJoin(pMs->tDbiThrdId);

    deleteAllElement(pMs->pClntInfo);

    deleteAllElement(pMs->pClntData);

    deleteAllElement(pMs->pSrvrCnfg);

    if(pMs->pCnd)
      gmlCdDstry(pMs->pCnd);

    if(pMs->pIfMtx)
      gmlMxDstry(pMs->pIfMtx);

    if(pMs->pDtMtx)
      gmlMxDstry(pMs->pDtMtx);
  }
}

inline int uTimeWait(const int iUSec)
{
  int iErr = 0;
  struct timeval tOut = {0, 0};

  tOut.tv_usec = iUSec;

  while((iErr = select(0, NULL, NULL, NULL, &tOut)))
    if(errno != EINTR)
      break;

  return iErr;
}

inline int uSockWait(const int iSock, const int iUSec)
{
  int iErr = 0;
  struct timeval tOut = {0, 0};
  fd_set  tRset, tWset;

  FD_ZERO(&tRset);
  FD_SET(iSock, &tRset);

  tWset = tRset;
  tOut.tv_usec = iUSec;

  while((iErr = select(iSock+1, &tRset, &tWset, NULL, &tOut)) == -1)
    if(errno != EINTR)
      break;

  return iErr;
}

inline static void freeEventMsg(void *pEvt)
{
  if(pEvt)
    free(((SNSREVENT *)pEvt)->pEventMsg);
}

void *doClientThread(void *pUser)
{
  CLIENTINF *pClntInf = pUser;
  GDATAPOLL *pSvdInfo = NULL;
  MONITSRVR *pMs = &tMonitSrv;
  SGNDCERT *pCert = NULL;
  SSL_CTX *pCtx = NULL;
  SSL *pCon = NULL;
  int iLocalFd = -1;
  char cLocalErr = 0;

  if(!pClntInf)
    return NULL;

  debugMessage("Enter client thread: %ld\n", gmlThreadId());

  iLocalFd = pClntInf->iSockFd;

  while(!pMs->cException && !cLocalErr)
  {
    int iOpt = 1;
    short sRSAKeyLn = 0;
    short sDayValid = 0;
    void *pBuff = malloc(PSEUDOHDLEN);

    sRSAKeyLn = getCnfgIntElement("RSA_key_length", pMs->pSrvrCnfg);
    if(sRSAKeyLn <= 0 || sRSAKeyLn > 0x7FFE)
      sRSAKeyLn = DFTRSAKEYLN;

    sDayValid = getCnfgIntElement("day_valid", pMs->pSrvrCnfg);
    if(sDayValid <= 0 || sDayValid > 0x7FFE)
      sDayValid = DFTDAYVALID;

    if(!(pCert = createX509Cert((RTCACERT *)pMs->pCert, sRSAKeyLn, sDayValid)))
    {
      pMs->cException = 1;
      errorMessage("createX509Cert fail ...\n");
      break;
    }

    if(!(pCtx = initServerCtx(pCert->pSignedCert, pCert->pPkey)))
    {
      pMs->cException = 1;
      errorMessage("initServerCtx fail ...\n");
      break;
    }

    debugMessage("SSL handshaking ...\n");

    if(!(pCon = acceptClient(pCtx, iLocalFd)))
    {
      cLocalErr = 1;
      errorMessage("acceptClient fail ...\n");
      break;
    }

    /* SSL handshake successed */
    /* Set the socket to nonblock mode */

    if(ioctl(iLocalFd, FIONBIO, &iOpt))
    {
      cLocalErr = 1;
      errorMessage("%s\n", strerror(errno));
      break;
    }

    pClntInf->ucStatus = SENSORUP;

    gmlMxLock(pMs->pIfMtx);
    pSvdInfo = newElement(pMs->pClntInfo);
    pSvdInfo->pAny = pClntInf;
    gmlMxUnLock(pMs->pIfMtx);

    while(!pMs->cException && !cLocalErr)
    {
      GDATAPOLL *pPool = NULL;
      int iLen = 0;
      unsigned long ulMsgID = 0;
      
      memset(pBuff, 0, PSEUDOHDLEN); 

      if((iLen = readSSL(pCon, (unsigned char *)pBuff, PSEUDOHDLEN)) == -1)
      {
        cLocalErr = 1;
        errorMessage("readSSL fail ...\n");
        break;
      }
      
      ulMsgID = *(unsigned long *)pBuff;

      if(ntohl(ulMsgID) == STATIMSG)
      {
        SNSRSTATS *pMsg = NULL;

        debugMessage("Statistic Message: %d\n", iLen);

        if(!(pMsg = (SNSRSTATS *)calloc(1, sizeof(SNSRSTATS))))
        {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
          errorMessage("%s\n", strerror(errno));
#else
          errorMessage("calloc failed.\n");
#endif
          break;
        }

        memcpy(pMsg, pBuff, PSEUDOHDLEN);

        iLen = readSSL(pCon, (unsigned char *)pMsg+PSEUDOHDLEN, STATSTUBLEN);

        if(iLen == -1)
        {
          cLocalErr = 1;
          errorMessage("readSSL fail ...\n");
          break;
        }

        if(!pClntInf->ulClntId)
        {
          GDATAPOLL *pCrntClnt = NULL;
          unsigned long ulSID = ntohl(pMsg->ulSensorId);

          gmlMxLock(pMs->pIfMtx);
          GETFIRST(pMs->pClntInfo, pCrntClnt);
#define CLNTINFO pCrntClnt->pNext->pAny
          while(pCrntClnt->pNext && CLNTINFO)
          {
            CLIENTINF *pCrnt = (CLIENTINF *)CLNTINFO;
#undef CLNTINFO

            if(pCrnt->ulClntId == ulSID) /* Sensor ID already in use. */
            {
              debugMessage("Sensor %ld has already connected ...\n", ulSID);
              cLocalErr = 1;
              break;
            }

            pCrntClnt=pCrntClnt->pNext;
          }
          gmlMxUnLock(pMs->pIfMtx);

          pClntInf->ulClntId = ulSID;
        }

        if(cLocalErr)
          break;

        gmlMxLock(pMs->pDtMtx);
        pPool = newElement(pMs->pClntData);
        gettimeofday(&pPool->tStamp, NULL);
        pPool->pAny = (void *)pMsg;
        gmlMxUnLock(pMs->pDtMtx);

        /* Send the control message to the remote. */
        iLen = writeSSL(pCon, &pClntInf->ucStatus, 1);

        if(iLen != 1)
        {
          cLocalErr = 1;
          errorMessage("writeSSL fail ...\n");
          break;
        }
      }
      else if(ntohl(ulMsgID) == EVENTMSG)
      {
        SNSREVENT *pEvt = NULL; 

        debugMessage("Event Message: %d\n", iLen);

        if(!(pEvt = (SNSREVENT *)calloc(1, sizeof(SNSREVENT))))
        {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
          errorMessage("%s\n", strerror(errno));
#else
          errorMessage("calloc failed.\n");
#endif
          break;
        }

        memcpy(pEvt, pBuff, PSEUDOHDLEN);

        if(ntohs(pEvt->usEvtLen))
        {
          pEvt->pEventMsg = calloc(1, ntohs(pEvt->usEvtLen)+1);

          iLen = readSSL(pCon, (unsigned char *)pEvt->pEventMsg,
                                                      ntohs(pEvt->usEvtLen));
          if(iLen == -1)
          {
            cLocalErr = 1;
            errorMessage("readSSL fail ...\n");
            break;
          }
        }

        /* Insert to pool */
        gmlMxLock(pMs->pDtMtx);
        pPool = newElement(pMs->pClntData);
        gettimeofday(&pPool->tStamp, NULL);
        if(pEvt->pEventMsg)
          pPool->pfAnyFree = freeEventMsg;
        pPool->pAny = (void *)pEvt;
        gmlMxUnLock(pMs->pDtMtx);
      }
      else
      {
        debugMessage("Unknown Message, ignoring ...\n");
        continue;
      }

      /* Signal the extract thread */
      gmlCdSignal(pMs->pCnd);
    } /* while */

    free(pBuff);
    break;
  }

  disconnectSSL(pCon, 1);
  cleanCtx(pCtx);
  freeSignedCert(pCert);

  if(iLocalFd != -1)
    close(iLocalFd);

  gmlMxLock(pMs->pIfMtx);
  deleteElement(pSvdInfo);
  gmlMxUnLock(pMs->pIfMtx);

  debugMessage("Leaving client thread ...\n");

  return NULL;
}

void *doExtractThread(void *pUser)
{
  MONITSRVR *pMs = pUser;

  if(!pMs)
    return NULL;

  debugMessage("Enter extracting thread: %ld\n", gmlThreadId());

  ++(pMs->ucActvClnts);

  monitGUI((void *)pMs);

  gmlThreadTerminate(pMs->tMainThdId);
  pMs->tExtThrdId = 0;
  debugMessage("Leaving extracting thread ...\n");
  return NULL;
}

int timeValComp(const struct timeval *pA, const struct timeval *pB)
{
  if(!pA && !pB)
    return 0;

  if(!pA && pB)
    return -1;

  if(pA && !pB)
    return 1;

  if(pA->tv_sec == pB->tv_sec)
  {
    if(pA->tv_usec == pB->tv_usec)
      return 0; /* A = B */
    else
      return (pA->tv_usec - pB->tv_usec > 0) ? 1 : -1; /* A > B or A < B */
  }
  else
    return (pA->tv_sec - pB->tv_sec > 0) ? 1 : -1; /* A > B or A < B */
}
