/*
   Copyright (C) 2000 - 2003 Min Hsu

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h> 

#include <gmlmutex.h>
#include <printmsg.h>

inline gmlMutex *gmlMxInit(void)
{
  gmlMutex *pMtx = (gmlMutex *)malloc(sizeof(gmlMutex));

  if(!pMtx)
  {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
    errorMessage("%s\n", strerror(errno));
#else
    errorMessage("pthread_mutex_init failed.\n");
#endif

    return NULL;
  }

  pthread_mutex_init(&pMtx->mutex, NULL);

  return pMtx;
}

inline void gmlMxDstry(gmlMutex *pMtx)
{
  if(pMtx)
  {
    pthread_mutex_destroy(&pMtx->mutex);
    free(pMtx);
  }
}

inline int gmlMxLock(gmlMutex *pMtx)
{
  if(pMtx)
    return pthread_mutex_lock(&pMtx->mutex);
  else
    return -1;
}

inline int gmlMxUnLock(gmlMutex *pMtx)
{
  if(pMtx)
    return pthread_mutex_unlock(&pMtx->mutex);
  else
    return -1;
}

inline int gmlMxTryLock(gmlMutex *pMtx)
{
  if(pMtx)
    return pthread_mutex_trylock(&pMtx->mutex);
  else
    return -1;
}

inline gmlCond *gmlCdInit(void)
{
  gmlCond *pCnd = (gmlCond *)malloc(sizeof(gmlCond));

  if(pCnd)
  {
    if(pthread_cond_init(&pCnd->cond, NULL))
    {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
      errorMessage("%s\n", strerror(errno));
#else
      errorMessage("pthread_cond_init failed.\n");
#endif
      free(pCnd);

      return NULL;
    }
  }

  return pCnd;
}

inline void gmlCdDstry(gmlCond *pCnd)
{
  if(pCnd)
  {
    pthread_cond_destroy(&pCnd->cond);
    free(pCnd);
  }
}

inline int gmlCdSignal(gmlCond *pCnd)
{
  if(pCnd)
    return pthread_cond_signal(&pCnd->cond);
  else
    return -1;
}

inline int gmlCdBroadcast(gmlCond *pCnd)
{
  if(pCnd)
    return pthread_cond_broadcast(&pCnd->cond);
  else
    return -1;
}

inline int gmlCdWait(gmlCond *pCnd, gmlMutex *pMtx)
{
  if(pCnd && pMtx)
    return pthread_cond_wait(&pCnd->cond, &pMtx->mutex);
  else
    return -1;
}

inline int gmlCdTimedWait(gmlCond *pCnd, gmlMutex *pMtx, unsigned long millis)
{
  int iErr=0;
  struct timeval tv;
  struct timespec ts;

  if(!pCnd || !pMtx)
    return -1;
  
  gettimeofday(&tv, NULL);

  ts.tv_sec = tv.tv_sec + (millis/1000);
  ts.tv_nsec = tv.tv_usec * 1000;

  while((iErr=pthread_cond_timedwait(&pCnd->cond, &pMtx->mutex, &ts))==EINTR);

  return iErr;
}
