/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <netinet/in.h>

#include <qapplication.h>
#include <qlist.h>
#include <qsplitter.h>
#include <qvbox.h>
#include <qstring.h>
#include <qpopupmenu.h>
#include <qiconview.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qtimer.h>

#include <gmlmonit.h>
#include <snortdata.h>
#include <dbprocs.h>

#include <eventview.h>
#include <sensor.xpm>
#include <httprfn.xpm>

EventView::EventView(void *pUser, QWidget *parent, const char *name) :
QWidget(parent, name)
{
  uiHiEvt = 0;
  uiMiEvt = 0;
  uiLoEvt = 0;
  uiNumInView = 0;
  ulPrevSid = 0;
  pRfnInfo = NULL;
  pListMenu = NULL;
  pMonit = pUser;

  tList.setAutoDelete(TRUE);

  QHBoxLayout *pHLay = new QHBoxLayout(this);
  pHLay->setMargin(3);

  QVBoxLayout *pVLay = new QVBoxLayout(pHLay);
  pVLay->setMargin(3);

  QSplitter *pHrzSplt = new QSplitter(Qt::Horizontal, this);
  pVLay->addWidget(pHrzSplt);

  pEvtTree = new QListView(pHrzSplt);
  pEvtList = new QListView(pHrzSplt);

  pEvtLabel = new QLabel(this);
  pEvtLabel->setMaximumHeight(16);
  pVLay->addWidget(pEvtLabel);

  pEvtTree->addColumn("Event Tree");
  pEvtTree->setRootIsDecorated(TRUE);
  
  connect(pEvtTree, SIGNAL(rightButtonPressed(QListViewItem *,
          const QPoint &, int)), this, SLOT(removeEvent(QListViewItem *,
          const QPoint &, int)));

  connect(pEvtTree, SIGNAL(expanded(QListViewItem *)), this,
                                       SLOT(nodeExpand(QListViewItem *)));

  connect(pEvtTree, SIGNAL(collapsed(QListViewItem *)), this,
                                       SLOT(nodeCollapse(QListViewItem *)));

  pEvtList->addColumn("Event Message");
  pEvtList->addColumn("Destination");
  pEvtList->addColumn("Source");
  pEvtList->addColumn("Destination MAC");
  pEvtList->addColumn("Source MAC");
  pEvtList->addColumn("Event Time");
  pEvtList->setAllColumnsShowFocus(TRUE);
  pEvtList->setShowSortIndicator(TRUE);
  pEvtList->setSorting(5, false);

  //pEvtList->setSelectionMode(QListView::Extended);
  connect(pEvtList, SIGNAL(rightButtonPressed(QListViewItem *,
          const QPoint &, int)), this, SLOT(processList(QListViewItem *,
          const QPoint &, int)));

  pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                             "Mid: %3    Low: %4           "
                             "            Total in view: %5").
                             arg(uiHiEvt+uiMiEvt+uiLoEvt).
                             arg(uiHiEvt).
                             arg(uiMiEvt).
                             arg(uiLoEvt).
                             arg(uiNumInView));
}

int EventView::searchTreeItem(ListItem *pItem, QListViewItem **pTrIm)
{
  ListItem *pList = NULL;
  int iLvl = 0;

  if(!pItem)
    return -1;

  for(pList=tList.first(); pList; pList=tList.next())
  {
    if(pList->tSnsrId == pItem->tSnsrId)
    {
      if(pList->tDstAddr == pItem->tDstAddr &&
         pList->tDstPort == pItem->tDstPort)
      {
        if(pList->tSrcAddr == pItem->tSrcAddr &&
           pList->tSrcPort == pItem->tSrcPort)
        {
          if(pList->tEvtDate == pItem->tEvtDate)
          {
            *pTrIm = pList->getTreeItem()->parent();
            iLvl = 4;
            break;
          }

          if(iLvl < 4)
          {
            *pTrIm = pList->getTreeItem()->parent()->parent();
            iLvl = 3;
          }
          continue;
        }

        if(iLvl < 3)
        {
          *pTrIm = pList->getTreeItem()->parent()->parent()->parent();
          iLvl = 2;
        }
        continue;
      }

      if(iLvl < 2)
      {
        *pTrIm = pList->getTreeItem()->parent()->parent()->parent()->parent();
        iLvl = 1;
      }
      continue;
    }
  }

  return iLvl;
}

void EventView::buildEvent(void *pEvt)
{
  int iLvl = 0;

  if(!pEvt)
    return;

  ListItem *pEvtItem = new ListItem((void *)pEvt);

  QListViewItem *pNewItem = NULL;

  iLvl = searchTreeItem(pEvtItem, &pNewItem);
  tList.append(pEvtItem);

  switch(iLvl)
  {
    case 0:
            pNewItem = new QListViewItem(pEvtTree, pEvtItem->tSnsrId);
            pNewItem->setPixmap(0, QPixmap(sensor_xpm));
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getDstAddr());
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getSrcAddr());
            pNewItem = new QListViewItem(pNewItem, pEvtItem->tEvtDate);
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getEvtTimeMsg());
            pEvtItem->setTreeItem(pNewItem);
            break;
    case 1:
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getDstAddr());
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getSrcAddr());
            pNewItem = new QListViewItem(pNewItem, pEvtItem->tEvtDate);
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getEvtTimeMsg());
            pEvtItem->setTreeItem(pNewItem);
            break;
    case 2:
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getSrcAddr());
            pNewItem = new QListViewItem(pNewItem, pEvtItem->tEvtDate);
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getEvtTimeMsg());
            pEvtItem->setTreeItem(pNewItem);
            break;
    case 3:
            pNewItem = new QListViewItem(pNewItem, pEvtItem->tEvtDate);
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getEvtTimeMsg());
            pEvtItem->setTreeItem(pNewItem);
            break;
    case 4:
            pNewItem = new QListViewItem(pNewItem, pEvtItem->getEvtTimeMsg());
            pEvtItem->setTreeItem(pNewItem);
  }

  if(pEvtTree->firstChild()->isOpen())
    addNewEvent(pEvtItem);

  switch(ntohs(((SNSREVENT *)pEvt)->usPriority))
  {
    case 1:
                ++uiHiEvt;
                break;
    case 2:
                ++uiMiEvt;
                break;
    default:
                ++uiLoEvt;
                break;
  }

  pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                             "Mid: %3    Low: %4           "
                             "            Total in view: %5").
	                     arg(uiHiEvt+uiMiEvt+uiLoEvt).
	                     arg(uiHiEvt).
                             arg(uiMiEvt).
                             arg(uiLoEvt).
                             arg(uiNumInView));

  removeAllItems(false);
}

void EventView::removeEvent(QListViewItem *pItem,
                            const QPoint &rPstn, int iVld)
{
  if(pItem && !iVld)
  {
    int iDelete = 0;
    int iExpand = 0;
    int iCollap = 0;

    QPopupMenu *pMenu = new QPopupMenu(pEvtTree);
    iDelete = pMenu->insertItem(QString("Remove"));
    //pMenu->insertSeparator();
    if(pItem->firstChild())
    {
      iExpand = pMenu->insertItem(QString("Expand"));
      iCollap = pMenu->insertItem(QString("Collapse"));
    }

    int iId = pMenu->exec(rPstn);

    if(iId == iDelete)
    {
      QListViewItem *pItemAbove = NULL;

      while(pItem)
      {
        pItemAbove = pItem->parent();

        if(pItemAbove)
        {
          if(pItemAbove->childCount() > 1)
          {
            removeListItem(pItem);
            delete pItem;
            pItem = NULL;
          }
          else
          {
            removeListItem(pItem);
            delete pItem;
            pItem = pItemAbove;
          }
        }
        else /* Root item */
        {
          removeListItem(pItem);
          delete pItem;
          pItem = NULL;
        }
      } /* while */

      pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                                 "Mid: %3    Low: %4           "
                                 "            Total in view: %5").
                                 arg(uiHiEvt+uiMiEvt+uiLoEvt).
                                 arg(uiHiEvt).
                                 arg(uiMiEvt).
                                 arg(uiLoEvt).
                                 arg(uiNumInView));
    }
    else if(iId == iExpand)
      pItem->setOpen(TRUE);
    else if(iId == iCollap)
      pItem->setOpen(FALSE);
  }
}

void EventView::nodeExpand(QListViewItem *pItem)
{
  EventListItem *pEvtItm = NULL;

  if(pItem && !pItem->parent())
  {
    ListItem *pList = NULL;

    for(pList=tList.first(); pList; pList=tList.next())
    {
      if(pItem->text(0).compare(pList->tSnsrId) == 0)
      {
        pEvtItm = new EventListItem(pEvtList, pList->tEvtMsg,
                                    pList->getDstAddrPort(),
                                    pList->getSrcAddrPort(),
                                    pList->tDstMac, pList->tSrcMac,
                                    pList->tEvtDate+" "+pList->tEvtTime);
        pEvtItm->setItemPriority(pList->getItemPriority());
        pList->setListItem(pEvtItm);

        ++uiNumInView;
      }
    }

    pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                               "Mid: %3    Low: %4           "
                               "            Total in view: %5").
                               arg(uiHiEvt+uiMiEvt+uiLoEvt).
                               arg(uiHiEvt).
                               arg(uiMiEvt).
                               arg(uiLoEvt).
                               arg(uiNumInView));
  }
}

void EventView::nodeCollapse(QListViewItem *pItem)
{
  if(pItem && !pItem->parent())
  {
    ListItem *pList = NULL;
    QListViewItemIterator itItm(pItem);

    for(pList=tList.first(); pList; pList=tList.next())
    {
      if(pItem->text(0).compare(pList->tSnsrId) == 0)
      {
        QListViewItem *pListItem = NULL;

        if((pListItem = pList->getListItem()))
        {
          delete pListItem;
          pList->setListItem(NULL);
        }
        if(uiNumInView)
          --uiNumInView;
      }
    }

    pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                               "Mid: %3    Low: %4           "
                               "            Total in view: %5").
                               arg(uiHiEvt+uiMiEvt+uiLoEvt).
                               arg(uiHiEvt).
                               arg(uiMiEvt).
                               arg(uiLoEvt).
                               arg(uiNumInView));
  }
}

static int referenceData(const unsigned char *pData,
                         const unsigned short usLen,
                         const void *pObj)
{
  EventView *pEvtVw = (EventView *)pObj;
  char *pRfnData = (char *)pData;

  if(pData && pObj)
  {
    char **pTmp = NULL;

    if(!pEvtVw->pRfnInfo)
    {
      pEvtVw->pRfnInfo = (char **)calloc((int)*pRfnData+1, sizeof(char *));
      pTmp = pEvtVw->pRfnInfo;
    }
    else
    {
      for(pTmp = pEvtVw->pRfnInfo; *pTmp; ++pTmp);
    }

    *pTmp = strdup(pRfnData+1);

    return 0;
  }

  return -1;
}

void EventView::processList(QListViewItem *pItem, const QPoint &rPstn, int iVld)
{
  if(pItem && iVld != -1)
  {
    int iDelete = 0;
    ListItem *pList = NULL;

    pListMenu = new QPopupMenu(pEvtList);
    iDelete = pListMenu->insertItem(QString("Remove"));

    //pMenu->insertSeparator();

    for(pList=tList.first(); pList; pList=tList.next())
    {
      if(pItem == pList->getListItem())
        break;
    }

    if(pList)
    {
      if(!((MONITSRVR *)pMonit)->uiNoDbs)
      {
        char **pTmp = pRfnInfo;

        if(pList->ulSigId != ulPrevSid)
        {
          ulPrevSid = pList->ulSigId;

          if(pRfnInfo) // Clean previous reference info.
          {
            while(*pTmp)
            {
              free(*pTmp);
              ++pTmp;
            }

            free(pRfnInfo);
            pRfnInfo = NULL;
          }

          QString tStmt;
          tStmt.sprintf("select httpReference from reference where "
                        "signatureId = %lu", pList->ulSigId);

          dbFetchReference(pMonit, tStmt.latin1(), referenceData, this);
        }

        if(pRfnInfo)
        {
          pTmp = pRfnInfo;

          QPopupMenu *pRfnMenu = new QPopupMenu(pEvtList);
          connect(pRfnMenu, SIGNAL(activated(int)), this,
                  SLOT(browseRfn(int)));

          while(*pTmp)
          {
            pRfnMenu->insertItem(QPixmap((const char **)httprfn), *pTmp);
            ++pTmp;
          }

          pListMenu->insertSeparator();
          pListMenu->insertItem("Reference", pRfnMenu);
        }
      }
    }

    int iId = pListMenu->exec(rPstn);

    if(iId == iDelete)
    {
      removeTreeItem(pItem);

      pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                                  "Mid: %3    Low: %4          "
                                  "           Total in view: %5").
                                   arg(uiHiEvt+uiMiEvt+uiLoEvt).
                                   arg(uiHiEvt).
                                   arg(uiMiEvt).
                                   arg(uiLoEvt).
                                   arg(uiNumInView));
    }
  }
}

void EventView::browseRfn(int iIdx)
{
  execBrowser(pListMenu->text(iIdx).latin1());
}

void EventView::execBrowser(const char *pTxt)
{
  char *pBrwsr = NULL;

  if(!pTxt)
    return;

  pBrwsr = getCnfgCharElement("browser", ((MONITSRVR *)pMonit)->pSrvrCnfg);

  if(!access(pBrwsr, R_OK|X_OK))
  {
    pid_t tPid;

    tPid = fork();

    if(!tPid)
    {
      if(execlp(pBrwsr, rindex(pBrwsr, '/')+1, pTxt, (char *)0) == -1)
      {
        exit(127);
      }
    }

    return;
  }

  QMessageBox::warning(this, "Browser", "Cannot Execute Browser!");
}

void EventView::addNewEvent(ListItem *pItem)
{
  EventListItem *pEvtItm = NULL;

  if(pItem)
  {
    pEvtItm = new EventListItem(pEvtList, pItem->tEvtMsg,
                                pItem->getDstAddrPort(),
                                pItem->getSrcAddrPort(),
                                pItem->tDstMac, pItem->tSrcMac,
				pItem->tEvtDate+" "+pItem->tEvtTime);
    pEvtItm->setItemPriority(pItem->getItemPriority());
    pItem->setListItem(pEvtItm);

    ++uiNumInView;
  }
}

void EventView::removeListItem(QListViewItem *pItem)
{
  short i = 0;
  ListItem *pList = NULL;
  QListViewItemIterator itItm(pItem);

  for(i=0; itItm.current(); ++i, ++itItm)
  {
    if(i && pItem->depth() >= itItm.current()->depth())
      break;

    if(!itItm.current()->firstChild())
    {
      for(pList=tList.first(); pList; pList=tList.next())
      {
        if(itItm.current() == pList->getTreeItem())
        {
          if(uiNumInView)
            --uiNumInView;
          delete pList->getListItem();
          tList.remove();
          break;
        }
      }
    }

    if(pItem->childCount()==0)
      break;
  }
}

void EventView::removeTreeItem(QListViewItem *pItem)
{
  ListItem *pList = NULL;

  for(pList=tList.first(); pList; pList=tList.next())
  {
    if(pItem == pList->getListItem())
    {
      QListViewItem *pItemAbove = NULL;

      pItem = pList->getTreeItem();

      while(pItem)
      {
        pItemAbove = pItem->parent();

        if(pItemAbove)
        {
          if(pItemAbove->childCount() > 1)
          {
            delete pItem;
            pItem = NULL;
          }
          else
          {
            delete pItem;
            pItem = pItemAbove;
          }
        }
        else /* Root item */
        {
          delete pItem;
          pItem = NULL;
        }
      } /* while */

      if(uiNumInView)
        --uiNumInView;
      delete pList->getListItem();
      tList.remove();
      break;
    }
  }
}

void EventView::removeAllItems(bool bAllItems)
{
  unsigned short usHi = 0;
  unsigned short usLo = 0;
  ListItem *pList = NULL;

  if(!bAllItems)
  {
    usHi = HIWATERMARK;
    usLo = LOWATERMARK;
  }

  if(tList.count() >= usHi)
  {
    for(pList=tList.first(); pList && tList.count()>=usLo;
                                                   pList=tList.next())
    {
      QListViewItem *pItemAbove = NULL;
      QListViewItem *pItem = pList->getTreeItem();

      while(pItem)
      {
        pItemAbove = pItem->parent();

        if(pItemAbove)
        {
          if(pItemAbove->childCount() > 1)
          {
            delete pItem;
            pItem = NULL;
          }
          else
          {
            delete pItem;
            pItem = pItemAbove;
          }
        }
        else /* Root item */
        {
          delete pItem;
          pItem = NULL;
        }
      } /* while */

      if(uiNumInView)
        --uiNumInView;
      delete pList->getListItem();
      tList.remove();
    }

    pEvtLabel->setText(QString("Event Total: %1    Hi: %2    "
                               "Mid: %3    Low: %4           "
                               "            Total in view: %5").
                               arg(uiHiEvt+uiMiEvt+uiLoEvt).
                               arg(uiHiEvt).
                               arg(uiMiEvt).
                               arg(uiLoEvt).
                               arg(uiNumInView));
  }
}

ListItem::ListItem(const void *pData)
{
  SNSREVENT *pEvt = (SNSREVENT *)pData;

  if(!pEvt)
    return;

  unsigned long ulDstIp = ntohl(pEvt->ulDstIp);
  unsigned long ulSrcIp = ntohl(pEvt->ulSrcIp);
  struct tm *pDate = localtime(&((pEvt->eventTime).tv_sec));

  tSnsrId = QString("Sensor ID: %1").arg(ntohl(pEvt->ulSensorId));

  if(pEvt->ucDstMac[0] || pEvt->ucDstMac[1] || pEvt->ucDstMac[2] ||
     pEvt->ucDstMac[3] || pEvt->ucDstMac[4] || pEvt->ucDstMac[5])
    tDstMac.sprintf("%02X:%02X:%02X:%02X:%02X:%02X",
                                  pEvt->ucDstMac[0], pEvt->ucDstMac[1],
                                  pEvt->ucDstMac[2], pEvt->ucDstMac[3],
                                  pEvt->ucDstMac[4], pEvt->ucDstMac[5]);
  else
    tDstMac.sprintf("N/A");

  if(pEvt->ucSrcMac[0] || pEvt->ucSrcMac[1] || pEvt->ucSrcMac[2] ||
     pEvt->ucSrcMac[3] || pEvt->ucSrcMac[4] || pEvt->ucSrcMac[5])
    tSrcMac.sprintf("%02X:%02X:%02X:%02X:%02X:%02X",
                                  pEvt->ucSrcMac[0], pEvt->ucSrcMac[1],
                                  pEvt->ucSrcMac[2], pEvt->ucSrcMac[3],
                                  pEvt->ucSrcMac[4], pEvt->ucSrcMac[5]);
  else
    tSrcMac.sprintf("N/A");

  if(ulDstIp)
    tDstAddr.sprintf("%ld.%ld.%ld.%ld", (ulDstIp>>24)&0xFF, (ulDstIp>>16)&0xFF,
                                        (ulDstIp>>8)&0xFF, ulDstIp&0xFF);
  else
    tDstAddr.sprintf("N/A");

  if(ulSrcIp)
    tSrcAddr.sprintf("%ld.%ld.%ld.%ld", (ulSrcIp>>24)&0xFF, (ulSrcIp>>16)&0xFF,
                                        (ulSrcIp>>8)&0xFF, ulSrcIp&0xFF);
  else
    tSrcAddr.sprintf("N/A");

  if(pEvt->usPrtclID == IPPROTO_TCP || pEvt->usPrtclID == IPPROTO_UDP)
  {
    tDstPort.sprintf("%d", ntohs(pEvt->usDstPort));
    tSrcPort.sprintf("%d", ntohs(pEvt->usSrcPort));
  }

  tEvtDate.sprintf("%d-%02d-%02d", 1900+pDate->tm_year,
                                   pDate->tm_mon+1,pDate->tm_mday);
  tEvtTime.sprintf("%02d:%02d:%02d", pDate->tm_hour,
                                     pDate->tm_min, pDate->tm_sec);
  tEvtMsg.sprintf("%s", pEvt->pEventMsg);

  ulSigId = ntohl(pEvt->ulSgntrId);
  usPriority = ntohs(pEvt->usPriority);
  usPrtclID = pEvt->usPrtclID;
  pTreeItem = NULL;
  pListItem = NULL;
}

EventListItem::EventListItem(QListView *parent, QString str1, QString str2,
                             QString str3, QString str4, QString str5,
                             QString str6, QString str7, QString str8)
: QListViewItem(parent)
{
  setText(0, str1);
  setText(1, str2);
  setText(2, str3);
  setText(3, str4);
  setText(4, str5);
  setText(5, str6);
  setText(6, str7);
  setText(7, str8);

  usPriority = 0;
}

void EventListItem::setItemPriority(const unsigned short usPr)
{
  usPriority = usPr;
}

void EventListItem::paintCell(QPainter *pPaint, const QColorGroup &rCg,
                              int iColumn, int iWidth, int iAlignment)
{
    QColorGroup tCg(rCg);
    QColor tC = tCg.text();

    switch(usPriority)
    {
      case 1: /* High Priority */
                 tCg.setColor(QColorGroup::Text, Qt::red);
                 break;
      case 2: /* Mid Priority */
                 tCg.setColor(QColorGroup::Text, Qt::magenta);
                 break;
      default: /* Low Priority */
                 tCg.setColor(QColorGroup::Text, Qt::blue);
                 break;
    }

    QListViewItem::paintCell(pPaint, tCg, iColumn, iWidth, iAlignment);

    tCg.setColor(QColorGroup::Text, tC);
}
