/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _EVENTVIEW_H_
#define _EVENTVIEW_H_

#include <sys/time.h>

#include <qobject.h>
#include <qwidget.h>
#include <qlist.h>
#include <qlabel.h>
#include <qlistview.h>

#include <statsview.h>

#define HIWATERMARK  4096
#define LOWATERMARK  3072

class EventListItem : public QListViewItem
{
public:
  EventListItem(QListView *parent, QString, QString = QString::null,
                QString = QString::null, QString = QString::null, 
                QString = QString::null, QString = QString::null, 
                QString = QString::null, QString = QString::null); 

  void setItemPriority(const unsigned short);
  void paintCell(QPainter *, const QColorGroup &, int, int, int);

private:
  unsigned short usPriority;
};

class ListItem
{
public:
  ListItem(const void *);
  ~ListItem() {}

  QString getDstAddrPort(void)
  {
    if(tDstPort.isNull())
      return tDstAddr;
    else
      return tDstAddr+":"+tDstPort;
  }
  
  QString getSrcAddrPort(void)
  {
    if(tSrcPort.isNull())
      return tSrcAddr;
    else
      return tSrcAddr+":"+tSrcPort;
  }
  
  QString getDstAddr(void)
  {
    return QString("Destination %1").arg(getDstAddrPort());
  }

  QString getSrcAddr(void)
  {
    return QString("Source %1").arg(getSrcAddrPort());
  }

  QString getEvtTimeMsg(void)
  {
    return tEvtTime+"        "+tEvtMsg;
  }

  void setTreeItem(QListViewItem *pItem)
  {
    pTreeItem = pItem;
  }

  QListViewItem *getTreeItem(void)
  {
    return pTreeItem;
  }

  void setListItem(QListViewItem *pItem)
  {
    pListItem = pItem;
  }

  QListViewItem *getListItem(void)
  {
    return pListItem;
  }

  unsigned short getItemPriority(void)
  {
    return usPriority;
  }

  QString tSnsrId;
  QString tDstMac;
  QString tSrcMac;
  QString tDstAddr;
  QString tSrcAddr;
  QString tDstPort;
  QString tSrcPort;
  QString tEvtDate;
  QString tEvtTime;
  QString tEvtMsg;

  unsigned long ulSigId;

protected:
  unsigned short usPrtclID;
  unsigned short usPriority;
  QListViewItem *pTreeItem;
  QListViewItem *pListItem;
};

class EventView : public QWidget
{
  Q_OBJECT

public:
  EventView(void *pUser = 0, QWidget *parent = 0, const char *name = 0);
  ~EventView()
  {
    removeAllItems(true);
    delete pEvtList;
    delete pEvtTree;
  }

  char **pRfnInfo;
  unsigned long ulPrevSid;

  void buildEvent(void *);

protected:
  QListView *pEvtTree;
  QListView *pEvtList;
  QLabel *pEvtLabel;
  QPopupMenu *pListMenu;
  unsigned int uiLoEvt;
  unsigned int uiMiEvt;
  unsigned int uiHiEvt;
  unsigned int uiNumInView;

  QList<ListItem> tList;

  void addNewEvent(ListItem *);
  int searchTreeItem(ListItem *, QListViewItem **);
  void removeListItem(QListViewItem *);
  void removeTreeItem(QListViewItem *);
  void removeAllItems(bool);

public slots:
  void removeEvent(QListViewItem *, const QPoint &, int);
  void nodeExpand(QListViewItem *);
  void nodeCollapse(QListViewItem *);
  void processList(QListViewItem *, const QPoint &, int);
  void execBrowser(const char *);
  void browseRfn(int);

private:
  void *pMonit;
};

#endif
