/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>

#include <qapplication.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qpixmap.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qfont.h>
#include <qplatinumstyle.h>

#include <gmlmonit.h>
#include <snortdata.h>
#include <netinet/in.h>

#include <monitgui.h>
#include <snort16.xpm>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

MonitGUI::MonitGUI(void *pUser, QWidget *parent, const char *name, WFlags f)
:QMainWindow(parent, name, f)
{
  pMonit = pUser;
  tTmStmp.tv_sec = 0;
  tTmStmp.tv_usec = 0;

  setIcon(QPixmap(snort_xpm));

  QTabWidget *pTab = new QTabWidget(this);
  QSplitter *pVtcSplt = new QSplitter(Qt::Vertical, pTab);
  pEventView = new EventView(pUser, pVtcSplt);
  pEventView->resize(640, 540);
  pStatsView = new StatsView(pUser, pVtcSplt);
  pTab->addTab(pVtcSplt, "Event");
  pTab->addTab(new Analysis(pUser, pTab), "Analysis");

  setCentralWidget(pTab);

  QPopupMenu *style = new QPopupMenu(this);
  style->setCheckable(TRUE);
  menuBar()->insertItem("&File" , style);

  style->insertItem("&Exit", qApp, SLOT(quit()), CTRL | Key_E);

  QPopupMenu *help = new QPopupMenu(this);
  menuBar()->insertSeparator();
  menuBar()->insertItem("&Help", help);
  help->insertItem("&About", this, SLOT(about()), Key_F1);

  QTimer *pTimer = new QTimer(this);
  pTimer->start(250);
  connect(pTimer, SIGNAL(timeout()), SLOT(readMsg()));

  qApp->setStyle(new QPlatinumStyle);
}

void MonitGUI::about()
{
  QString strAbout = QString("Snort Monitor Version: %1").arg(VERSION); 

  QMessageBox::about(this, strAbout, 
			"<p>A Qt based GUI to monitor multiple "
                        "<b>Snort Sensors</b> in a centralized "
                        "monitor screen."
                        "<br><i> -- Min Hsu</i></p>");
}

void MonitGUI::readMsg(void)
{
  if(!pMonit)
    return;

  MONITSRVR *pMs = (MONITSRVR *)pMonit;
  GDATAPOLL *pFirst = NULL;
  GETFIRST(pMs->pClntData, pFirst);

  gmlMxLock(pMs->pDtMtx);
#define CLNTDATA pFirst->pNext->pAny
  while(pFirst && pFirst->pNext && CLNTDATA)
  {
    unsigned long ulMsgID = *(unsigned long *)CLNTDATA;

    if(timeValComp(&pFirst->pNext->tStamp, &tTmStmp)<=0)
    {
      pFirst = pFirst->pNext;
      continue;
    }

    if(ntohl(ulMsgID) == EVENTMSG)
    {
      pEventView->buildEvent((void *)CLNTDATA);
    }
    else if(ntohl(ulMsgID) == STATIMSG)
    {
      pStatsView->insertSttData((void *)CLNTDATA);
    }
#undef CLNTDATA

    tTmStmp.tv_sec = pFirst->pNext->tStamp.tv_sec;
    tTmStmp.tv_usec = pFirst->pNext->tStamp.tv_usec;

    ++(pFirst->pNext->uiRfrncd);

    pFirst = pFirst->pNext;
  } /* while */
  gmlMxUnLock(pMs->pDtMtx);
}

#ifdef __cplusplus
extern "C" {
#endif

int monitGUI(void *pUser)
{
  int argc = 0;
  char **argv = NULL;

  QApplication::setColorSpec(QApplication::ManyColor);
  QFont tFnt = QApplication::font();
  if(tFnt.pointSize() > 11)
    QApplication::setFont(QFont("Helvetica", 11));
  QApplication *pApp = new QApplication(argc, argv);

  MonitGUI *pMonit = new MonitGUI(pUser);

  QString strCpt = QString("Monitor Version: %1").arg(VERSION);
  pMonit->setCaption(strCpt);
  pMonit->resize(640, 720);
  pApp->setMainWidget(pMonit);
  pMonit->show();
  pApp->setMainWidget(0);

  pApp->connect(pApp, SIGNAL(lastWindowClosed()), pApp, SLOT(quit()));

  argc = pApp->exec();

  delete pMonit;
  delete pApp;

  return argc;
}

#ifdef __cplusplus
}
#endif
