/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <qvariant.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qtimer.h>

#include <gmlmonit.h>
#include <dbprocs.h>
#include <report.h>
#include <snort16.xpm>
#include <httprfn.xpm>

#define PAGESIZE  20
#define RPTTBLHDR "<html><body bgcolor=\"#ffffff\"><div align=\"center\"><p>\
<i><font size=\"+3\" face=\"Times New Roman, Times, serif\">\
<b><font color=\"#3333ff\">REPORT</font></b></font><br>\
</i></p></div><br><br>"
#define DTLTBLHDR "<table align=\"center\" bgcolor=\"#ffffff\" width=\"100%\" \
border=\"1\" cellspacing=\"0\" cellpadding=\"0\"><tbody>\
<tr><td><small>Sensor Id<br></small></td>\
<td><small>Eevent Time</small><br></td>\
<td><small>Event Message<br></small></td>\
<td><small>Source IP<br></small></td>\
<td><small>Source Port<br></small></td>\
<td><small>Destination IP<br></small></td>\
<td><small>Destination Port<br></small></td>\
<td><small>Priority<br></small></td>\
<td><small>Protocol<br></small></td>\
<td><small>Source Mac<br></small></td>\
<td><small>Destination Mac<br></small></td></tr>"

Analysis::Analysis(void *pUser, QWidget* parent, const char* name) :
QWidget(parent, name)
{
  pMonit = pUser;
  iRowsInTbl = 0;
  ulPrevSid = 0;
  pRfnInfo = NULL;
  pInfoMenu = NULL;
  pDtl = NULL;

  if(!name)
    setName("Analysis");

  pLayout = new QGridLayout(this, 1, 1, 6, 6, "pLayout"); 

  snsrId = new QLabel(this, "snsrId");
  snsrId->setText(tr("Sensor ID"));
  pLayout->addWidget(snsrId, 0, 0);

  pSnsrId = new QLineEdit(this, "pSnsrId");
  pLayout->addMultiCellWidget(pSnsrId, 0, 0, 1, 2);

  sigId = new QLabel(this, "sigId");
  sigId->setText(tr("Signature ID"));
  pLayout->addMultiCellWidget(sigId, 0, 0, 9, 10);

  pSigId = new QLineEdit(this, "pSigId");
  pLayout->addWidget(pSigId, 0, 11);

  source = new QLabel(this, "source");
  source->setText(tr("Source"));
  pLayout->addWidget(source, 1, 0);

  srcIpFrom = new QLabel(this, "srcIpFrom");
  srcIpFrom->setText(tr("IP From"));
  srcIpFrom->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  pLayout->addMultiCellWidget(srcIpFrom, 1, 1, 1, 2);

  pSrcIpFrom = new QLineEdit(this, "pSrcIpFrom");
  pSrcIpFrom->setMinimumWidth(110);
  pLayout->addWidget(pSrcIpFrom, 1, 3);

  srcIpTo = new QLabel(this, "srcIpTo");
  srcIpTo->setText(tr("To"));
  pLayout->addWidget(srcIpTo, 1, 4);

  pSrcIpTo = new QLineEdit(this, "pSrcIpTo");
  pSrcIpTo->setMinimumWidth(110);
  pLayout->addMultiCellWidget(pSrcIpTo, 1, 1, 5, 7);

  srcPtFrom = new QLabel(this, "srcPtFrom");
  srcPtFrom->setText(tr("Port From"));
  srcPtFrom->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  pLayout->addWidget(srcPtFrom, 1, 8);

  pSrcPtFrom = new QLineEdit(this, "pSrcPtFrom");
  pLayout->addWidget(pSrcPtFrom, 1, 9);

  srcPtTo = new QLabel(this, "srcPtTo");
  srcPtTo->setText(tr("To"));
  pLayout->addWidget(srcPtTo, 1, 10);

  pSrcPtTo = new QLineEdit(this, "pSrcPtTo");
  pLayout->addWidget(pSrcPtTo, 1, 11);

  destination = new QLabel(this, "destination");
  destination->setText(tr("Destination"));
  pLayout->addWidget(destination, 2, 0);

  dstIpFrom = new QLabel(this, "dstIpFrom");
  dstIpFrom->setText(tr("IP From"));
  dstIpFrom->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  pLayout->addMultiCellWidget(dstIpFrom, 2, 2, 1, 2);

  pDstIpFrom = new QLineEdit(this, "pDstIpFrom");
  pLayout->addWidget(pDstIpFrom, 2, 3);

  dstIpTo = new QLabel(this, "dstIpTo");
  dstIpTo->setText(tr("To"));
  pLayout->addWidget(dstIpTo, 2, 4);

  pDstIpTo = new QLineEdit(this, "pDstIpTo");
  pLayout->addMultiCellWidget(pDstIpTo, 2, 2, 5, 7);

  dstPtFrom = new QLabel(this, "dstPtFrom");
  dstPtFrom->setText(tr("Port From"));
  dstPtFrom->setAlignment(int(QLabel::AlignVCenter | QLabel::AlignRight));
  pLayout->addWidget(dstPtFrom, 2, 8);

  pDstPtFrom = new QLineEdit(this, "pDstPtFrom");
  pLayout->addWidget(pDstPtFrom, 2, 9);

  dstPtTo = new QLabel(this, "dstPtTo");
  dstPtTo->setText(tr("To"));
  pLayout->addWidget(dstPtTo, 2, 10);

  pDstPtTo = new QLineEdit(this, "pDstPtTo");
  pLayout->addWidget(pDstPtTo, 2, 11);

  startDT = new QLabel(this, "startDT");
  startDT->setText(tr("Starting Date Time"));
  pLayout->addMultiCellWidget(startDT, 3, 3, 0, 1);

  pStartDT = new QDateTimeEdit(this, "pStartDT");
  pLayout->addMultiCellWidget(pStartDT, 3, 3, 2, 5);

  endDT = new QLabel(this, "endDT");
  endDT->setText(tr("Ending Date Time"));
  pLayout->addMultiCellWidget(endDT, 4, 4, 0, 1);

  pEndDT = new QDateTimeEdit(this, "pEndDT");
  pLayout->addMultiCellWidget(pEndDT, 4, 4, 2, 5);

  protocol = new QLabel(this, "protocol");
  protocol->setText(tr("Protocol"));
  pLayout->addWidget(protocol, 3, 6);

  pProtocol = new QComboBox(FALSE, this, "pProtocol");
  pProtocol->insertItem(tr("ALL"));
  pProtocol->insertItem(tr("IP"));
  pProtocol->insertItem(tr("TCP"));
  pProtocol->insertItem(tr("UDP"));
  pProtocol->insertItem(tr("ICMP"));
  pLayout->addMultiCellWidget(pProtocol, 3, 3, 7, 8);

  priority = new QLabel(this, "priority");
  priority->setText(tr("Priority"));
  pLayout->addWidget(priority, 4, 6);

  pPriority = new QComboBox(FALSE, this, "pPriority");
  pPriority->insertItem(tr("ALL"));
  pPriority->insertItem(tr("HIGH"));
  pPriority->insertItem(tr("MEDIUM"));
  pPriority->insertItem(tr("LOW"));
  pLayout->addMultiCellWidget(pPriority, 4, 4, 7, 8);

  pAnalysis = new QPushButton(this, "pAnalysis");
  pAnalysis->setText(tr("Analysis"));
  pLayout->addMultiCellWidget(pAnalysis, 3, 3, 9, 11);

  connect(pAnalysis, SIGNAL(clicked()), this, SLOT(genAnalysis()));

  pReport = new QPushButton(this, "pReport");
  pReport->setText(tr("Report"));
  pLayout->addMultiCellWidget(pReport, 4, 4, 9, 11);

  connect(pReport, SIGNAL(clicked()), this, SLOT(genReport()));

  pRsltTbl = new QTable(this, "pRsltTbl");
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Signature ID"), 80);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Event"), 204);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Percentage"), 76);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Prority"), 48);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Attack"), 52);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Attacker"), 66);
  pRsltTbl->setNumCols(pRsltTbl->numCols() + 1);
  pRsltTbl->horizontalHeader()->setLabel(pRsltTbl->numCols() - 1,
                       tr("Attackee"), 64);
  pRsltTbl->setNumCols(7);

  pRsltTbl->setReadOnly(TRUE);
  pRsltTbl->setSorting(FALSE);
  pRsltTbl->setSelectionMode(QTable::MultiRow);
  pRsltTbl->setFocusStyle(QTable::FollowStyle);
  pLayout->addMultiCellWidget(pRsltTbl, 5, 5, 0, 11);

  connect(pRsltTbl, SIGNAL(pressed(int, int, int, const QPoint &)),
                  this, SLOT(moreInfo(int, int, int, const QPoint &)));

  connect(pRsltTbl, SIGNAL(doubleClicked(int, int, int, const QPoint &)),
                  this, SLOT(genDetail(int, int, int, const QPoint &)));

  iSumAtcks = 0;
  iSumAtckr = 0;
  iSumAtckd = 0;
  pSId = NULL;
}

int Analysis::genSqlStmt(QString &rStmt)
{
  bool bNoCndt = false;
  tFchCndtn.truncate(0);

  rStmt = "select signatureId, priority, count(*), eventMsg from events ";

  if(pSrcIpFrom->text().length() || pSrcIpTo->text().length() ||
     pDstIpFrom->text().length() || pDstIpTo->text().length())
  {
    rStmt += "a, addrinfo b ";
  }

  if(pSnsrId->text().length())
  {
    long lTmp = 0;

    lTmp = atol(pSnsrId->text().latin1());

    if(lTmp < 0 || (!lTmp && *(pSnsrId->text().latin1()) != '0'))
    {
      QMessageBox::warning(this, "Sensor ID",
                                 "Invalid Sensor ID: "
                                 +pSnsrId->text());
       return -1;
    }

    rStmt += "where sensorId = "+pSnsrId->text();
    tFchCndtn = "and sensorId = "+pSnsrId->text();
  }
  else
    bNoCndt = true;

  if(pSigId->text().length())
  {
    int iTmp = 0;

    iTmp = atoi(pSigId->text().latin1());

    if(iTmp < 0 || iTmp > 0xffff || 
       (!iTmp && *(pSigId->text().latin1()) != '0'))
    {
      QMessageBox::warning(this, "Signature ID",
                                 "Invalid Signature ID: "
                                 +pSigId->text());
      return -1;
    }

    if(bNoCndt)
    {
      rStmt += "where signatureId = "+pSigId->text();
      bNoCndt = false;
    }
    else
      rStmt += " and signatureId = "+pSigId->text();
  }

  if(pSrcIpFrom->text().length() || pSrcIpTo->text().length() ||
     pDstIpFrom->text().length() || pDstIpTo->text().length())
  {
    struct in_addr tAddr = {0};
    unsigned long ulIpAddr = 0;

    if(bNoCndt)
    {
      rStmt += "where a.addrId = b.addrId";
      bNoCndt = false;
    }
    else
      rStmt += " and a.addrId = b.addrId";

    if(pSrcIpFrom->text().length())
    {
      if(!inet_aton(pSrcIpFrom->text().latin1(), &tAddr))
      {
        QMessageBox::warning(this, "Source IP Address",
                                   "Invalid IP Address: "
                                   +pSrcIpFrom->text());
        return -1;
      }

      ulIpAddr = ntohl(tAddr.s_addr);
      rStmt += " and srcIp >= "+QString("%1").arg(ulIpAddr);
      tFchCndtn += " and srcIp >= "+QString("%1").arg(ulIpAddr);
    }

    tAddr.s_addr = 0;

    if(pSrcIpTo->text().length())
    {
      if(!inet_aton(pSrcIpTo->text().latin1(), &tAddr))
      {
        QMessageBox::warning(this, "Source IP Address",
                                   "Invalid IP Address: "
                                   +pSrcIpTo->text());
        return -1;
      }

      ulIpAddr = ntohl(tAddr.s_addr);
      rStmt += " and srcIp <= "+QString("%1").arg(ulIpAddr);
      tFchCndtn += " and srcIp <= "+QString("%1").arg(ulIpAddr);
    }

    tAddr.s_addr = 0;

    if(pDstIpFrom->text().length())
    {
      if(!inet_aton(pDstIpFrom->text().latin1(), &tAddr))
      {
        QMessageBox::warning(this, "Destination IP Address",
                                   "Invalid IP Address: "
                                   +pDstIpFrom->text());
        return -1;
      }

      ulIpAddr = ntohl(tAddr.s_addr);
      rStmt += " and dstIp >= "+QString("%1").arg(ulIpAddr);
      tFchCndtn += " and dstIp >= "+QString("%1").arg(ulIpAddr);
    }

    if(pDstIpTo->text().length())
    {
      if(!inet_aton(pDstIpTo->text().latin1(), &tAddr))
      {
        QMessageBox::warning(this, "Destination IP Address",
                                   "Invalid IP Address: "
                                   +pDstIpTo->text());
        return -1;
      }

      ulIpAddr = ntohl(tAddr.s_addr);
      rStmt += " and dstIp <= "+QString("%1").arg(ulIpAddr);
      tFchCndtn += " and dstIp <= "+QString("%1").arg(ulIpAddr);
    }
  }

  if(pSrcPtFrom->text().length() || pSrcPtTo->text().length() ||
     pDstPtFrom->text().length() || pDstPtTo->text().length())
  {
    int iTmp = 0;

    if(pSrcPtFrom->text().length())
    {
      iTmp = atoi(pSrcPtFrom->text().latin1());

      if(iTmp < 0 || iTmp > 0xffff ||
         (!iTmp && *(pSrcPtFrom->text().latin1()) != '0'))
      {
        QMessageBox::warning(this, "Source Port",
                                   "Invalid Port: "
                                   +pSrcPtFrom->text());
        return -1;
      }

      if(bNoCndt)
      {
        rStmt += "where srcPort >= "+pSrcPtFrom->text();
        bNoCndt = false;
      }
      else
        rStmt += " and srcPort >= "+pSrcPtFrom->text();

      tFchCndtn += " and srcPort >= "+pSrcPtFrom->text();
    }

    if(pSrcPtTo->text().length())
    {
      iTmp = atoi(pSrcPtTo->text().latin1());

      if(iTmp < 0 || iTmp > 0xffff ||
         (!iTmp && *(pSrcPtTo->text().latin1()) != '0'))
      {
        QMessageBox::warning(this, "Source Port",
                                   "Invalid Port: "
                                   +pSrcPtTo->text());
        return -1;
      }

      if(bNoCndt)
      {
        rStmt += "where srcPort <= "+pSrcPtTo->text();
        bNoCndt = false;
      }
      else
        rStmt += " and srcPort <= "+pSrcPtTo->text();

      tFchCndtn += " and srcPort <= "+pSrcPtTo->text();
    }

    if(pDstPtFrom->text().length())
    {
      iTmp = atoi(pDstPtFrom->text().latin1());

      if(iTmp < 0 || iTmp > 0xffff ||
         (!iTmp && *(pDstPtFrom->text().latin1()) != '0'))
      {
        QMessageBox::warning(this, "Destination Port",
                                   "Invalid Port: "
                                   +pDstPtFrom->text());
        return -1;
      }

      if(bNoCndt)
      {
        rStmt += "where dstPort >= "+pDstPtFrom->text();
        bNoCndt = false;
      }
      else
        rStmt += " and dstPort >= "+pDstPtFrom->text();

      tFchCndtn += " and dstPort >= "+pDstPtFrom->text();
    }

    if(pDstPtTo->text().length())
    {
      iTmp = atoi(pDstPtTo->text().latin1());

      if(iTmp < 0 || iTmp > 0xffff ||
         (!iTmp && *(pDstPtTo->text().latin1()) != '0'))
      {
        QMessageBox::warning(this, "Destination Port",
                                   "Invalid Port: "
                                   +pDstPtTo->text());
        return -1;
      }

      if(bNoCndt)
      {
        rStmt += "where dstPort <= "+pDstPtTo->text();
        bNoCndt = false;
      }
      else
        rStmt += " and dstPort <= "+pDstPtTo->text();

      tFchCndtn += " and dstPort <= "+pDstPtTo->text();
    }
  }

  if(pProtocol->currentText() != "ALL")
  {
    if(bNoCndt)
    {
      rStmt += " where protocolID = ";
      bNoCndt = false;
    }
    else
      rStmt += " and protocolID = ";

    if(pProtocol->currentText() == "ICMP")
    {
      rStmt += "1";
      tFchCndtn += " and protocolID = 1"; 
    }
    else if(pProtocol->currentText() == "TCP")
    {
      rStmt += "6";
      tFchCndtn += " and protocolID = 6"; 
    }
    else if(pProtocol->currentText() == "UDP")
    {
      rStmt += "17";
      tFchCndtn += " and protocolID = 17"; 
    }
    else
    {
      rStmt += "0";
      tFchCndtn += " and protocolID = 0"; 
    }
  }

  if(pPriority->currentText() != "ALL")
  {
    if(bNoCndt)
    {
      rStmt += "where priority ";
      bNoCndt = false;
    }
    else
      rStmt += " and priority ";

    if(pPriority->currentText() == "HIGH")
    {
      rStmt += "= 1";
      tFchCndtn += " and priority = 1";
    }
    else if(pPriority->currentText() == "MEDIUM")
    {
      rStmt += "= 2";
      tFchCndtn += " and priority = 2";
    }
    else
    {
      rStmt += ">= 3";
      tFchCndtn += " and priority >= 3";
    }
  }

  struct tm tDT;
  time_t tc = time(0);
  unsigned long ulSec = 0;

  if(pStartDT->dateTime().isValid())
  {
    memset(&tDT, 0, sizeof(tDT));
    tDT.tm_year = pStartDT->dateTime().date().year()-1900;
    tDT.tm_mon = pStartDT->dateTime().date().month()-1;
    tDT.tm_mday = pStartDT->dateTime().date().day();
    tDT.tm_hour = pStartDT->dateTime().time().hour();
    tDT.tm_min = pStartDT->dateTime().time().minute();
    tDT.tm_sec = pStartDT->dateTime().time().second();
    tDT.tm_isdst = (localtime(&tc))->tm_isdst;

    ulSec = (unsigned long)mktime(&tDT);

    if(bNoCndt)
    {
      rStmt += "where eventTime >= "+QString("%1").arg(ulSec);
      bNoCndt = false;
    }
    else
      rStmt += " and eventTime >= "+QString("%1").arg(ulSec);

    tFchCndtn += " and eventTime >= "+QString("%1").arg(ulSec);
  }

  if(pEndDT->dateTime().isValid())
  {
    memset(&tDT, 0, sizeof(tDT));
    tDT.tm_year = pEndDT->dateTime().date().year()-1900;
    tDT.tm_mon = pEndDT->dateTime().date().month()-1;
    tDT.tm_mday = pEndDT->dateTime().date().day();
    tDT.tm_hour = pEndDT->dateTime().time().hour();
    tDT.tm_min = pEndDT->dateTime().time().minute();
    tDT.tm_sec = pEndDT->dateTime().time().second();
    tDT.tm_isdst = (localtime(&tc))->tm_isdst;

    ulSec = (unsigned long)mktime(&tDT);

    if(bNoCndt)
    {
      rStmt += "where eventTime <= "+QString("%1").arg(ulSec);
      bNoCndt = false;
    }
    else
      rStmt += " and eventTime <= "+QString("%1").arg(ulSec);

    tFchCndtn += " and eventTime <= "+QString("%1").arg(ulSec);
  }

  rStmt += " group by signatureId order by 3 desc";
  tFchCndtn += " and";

  return 0;
}

void Analysis::buildAnalysis(const void *pData)
{
  ATCKINFO *pAtckIf = (ATCKINFO *)pData;
  char *pEvtMsg = (char *)pData+ATCKIFLEN;

  if(!pData)
    return;

  ++iRowsInTbl;
  QString tHdLbl;
  tHdLbl.sprintf("%d", iRowsInTbl);
  QString tPrcnt;
  tPrcnt.sprintf("%4.1f%%", 100*(1.0*pAtckIf->uiAtcks/pAtckIf->uiAtckt));

  pRsltTbl->setNumRows(iRowsInTbl);
  pRsltTbl->verticalHeader()->setLabel(iRowsInTbl, tr(tHdLbl));
  pRsltTbl->setText(iRowsInTbl-1, 0, QString("%1").arg(pAtckIf->usSigId));
  pRsltTbl->setText(iRowsInTbl-1, 1, pEvtMsg);
  pRsltTbl->setText(iRowsInTbl-1, 2, tPrcnt);

  if(pAtckIf->sPrrty==1)
  {
    QTableItem *pPrt = new SetPriority(pRsltTbl, QTableItem::Never,
                                      "High", pAtckIf->sPrrty);
    pRsltTbl->setItem(iRowsInTbl-1, 3, pPrt); 
  }
  else if(pAtckIf->sPrrty==2)
  {
    QTableItem *pPrt = new SetPriority(pRsltTbl, QTableItem::Never,
                                      "Medium", pAtckIf->sPrrty);
    pRsltTbl->setItem(iRowsInTbl-1, 3, pPrt);
  }
  else
  {
    QTableItem *pPrt = new SetPriority(pRsltTbl, QTableItem::Never,
                                      "Low", pAtckIf->sPrrty);
    pRsltTbl->setItem(iRowsInTbl-1, 3, pPrt);
  }

  pRsltTbl->setText(iRowsInTbl-1, 4, QString("%1").arg(pAtckIf->uiAtcks));
  pRsltTbl->setText(iRowsInTbl-1, 5, QString("%1").arg(pAtckIf->uiAtckrs));
  pRsltTbl->setText(iRowsInTbl-1, 6, QString("%1").arg(pAtckIf->uiAtckd));

  free(pAtckIf);
}

static int analysisData(const unsigned char *pData,
                       const unsigned short usLen,
                       const void *pObj)
{
  Analysis *pAnls = (Analysis *)pObj;
  ATCKINFO *pAtckInfo = (ATCKINFO *)pData;
  unsigned short usDataLen = 0;

  if(pData && pObj)
  {
    usDataLen = (unsigned short)ntohl(pAtckInfo->uiMsgLen)+ATCKIFLEN;

    if(usDataLen != usLen)
      return -1;

    pAnls->buildAnalysis(pData);
    return 0;
  }

  return -1;
}

void Analysis::genAnalysis(void)
{
  QString tSqlStmt;

  if(((MONITSRVR *)pMonit)->uiNoDbs)
  {
    QMessageBox::warning(this, "DataBase Status",
                               "Can Not Connect to DataBase");
    return;
  }

  if(genSqlStmt(tSqlStmt))
    return;

  iRowsInTbl = 0;
  pRsltTbl->setNumRows(iRowsInTbl);
  dbFetchAtckInfo(pMonit, tSqlStmt.latin1(), analysisData, this);
}

void Analysis::genCondition(QString &rCndtn)
{
  if(pSnsrId->text().length())
    rCndtn += "Sensor ID <small>"+pSnsrId->text()+"</small><br>";

  if(pSigId->text().length())
    rCndtn += "Signature ID <small>"+pSigId->text()+"</small><br>";

  if(pSrcIpFrom->text().length() && pSrcIpTo->text().length())
  {
    if(pSrcIpFrom->text() == pSrcIpTo->text())
      rCndtn += "Source IP <small>"+pSrcIpFrom->text()+"</small><br>";
    else
    {
      rCndtn += "Source IP <small>&gt;="+pSrcIpFrom->text();
      rCndtn +=", &lt;="+pSrcIpTo->text()+"</small><br>";
    }
  }
  else if(pSrcIpFrom->text().length())
    rCndtn += "Source IP <small>&gt;="+pSrcIpFrom->text();
  else if(pSrcIpTo->text().length())
    rCndtn += "Source IP <small>&lt;="+pSrcIpTo->text();
      
  if(pSrcPtFrom->text().length() && pSrcPtTo->text().length())
  {
    if(pSrcPtFrom->text() == pSrcPtTo->text())
      rCndtn += "Source Port <small>"+pSrcPtFrom->text()+"</small><br>";
    else
    {
      rCndtn += "Source Port <small>&gt;="+pSrcPtFrom->text();
      rCndtn +=", &lt;="+pSrcPtTo->text()+"</small><br>";
    }
  }
  else if(pSrcPtFrom->text().length())
    rCndtn += "Source Port <small>&gt;="+pSrcPtFrom->text();
  else if(pSrcPtTo->text().length())
    rCndtn += "Source Port <small>&lt;="+pSrcPtTo->text();

  if(pDstIpFrom->text().length() && pDstIpTo->text().length())
  {
    if(pDstIpFrom->text() == pDstIpTo->text())
      rCndtn += "Destination IP <small>"+pDstIpFrom->text()+"</small><br>";
    else
    {
      rCndtn += "Destination IP <small>&gt;="+pDstIpFrom->text();
      rCndtn +=", &lt;="+pDstIpTo->text()+"</small><br>";
    }
  }
  else if(pDstIpFrom->text().length())
    rCndtn += "Destination IP <small>&gt;="+pDstIpFrom->text();
  else if(pDstIpTo->text().length())
    rCndtn += "Destination IP <small>&lt;="+pDstIpTo->text();
      
  if(pDstPtFrom->text().length() && pDstPtTo->text().length())
  {
    if(pDstPtFrom->text() == pDstPtTo->text())
      rCndtn += "Destination Port <small>"+pDstPtFrom->text()+"</small><br>";
    else
    {
      rCndtn += "Destination Port <small>&gt;="+pDstPtFrom->text();
      rCndtn +=", &lt;="+pDstPtTo->text()+"</small><br>";
    }
  }
  else if(pDstPtFrom->text().length())
    rCndtn += "Destination Port <small>&gt;="+pDstPtFrom->text();
  else if(pDstPtTo->text().length())
    rCndtn += "Destination Port <small>&lt;="+pDstPtTo->text();

  if(pProtocol->currentText() != "ALL")
    rCndtn += "Protocol <small>"+pProtocol->currentText()+"</small><br>";

  if(pPriority->currentText() != "ALL")
    rCndtn += "Priority <small>"+pPriority->currentText()+"</small><br>";

  if(pStartDT->dateTime().isValid() && pEndDT->dateTime().isValid())
  {
    if(pStartDT->dateTime() == pEndDT->dateTime())
      rCndtn += "Event Time <small>"+pStartDT->dateTime().toString();
    else
    {
      rCndtn += "Event Time <small>&gt;="+pStartDT->dateTime().toString();
      rCndtn += " &lt;="+pEndDT->dateTime().toString();
    }
    rCndtn += "</small><br>";
  }
  else if(pStartDT->dateTime().isValid())
  {
    rCndtn += "Event Time <small>&gt;="+pStartDT->dateTime().toString();
    rCndtn += "</small><br>";
  }
  else if(pEndDT->dateTime().isValid())
  {
    rCndtn += "Event Time <small>&lt;="+pEndDT->dateTime().toString();
    rCndtn += "</small><br>";
  }

  rCndtn += "<br><table align=\"center\" cellpadding=\"0\" ";
  rCndtn += "cellspacing=\"0\" width=\"100%\" border=\"0\" ";
  rCndtn += "bgcolor=\"white\"><tbody>";
  rCndtn += "<tr><td width=\"12%\">Signature ID</td>";
  rCndtn += "<td width=\"28%\">Event Message</td>";
  rCndtn += "<td width=\"22%\">Percentage</td>";
  rCndtn += "<td width=\"5%\"></td><td width=\"8%\">Priority</td>";
  rCndtn += "<td width=\"7%\">Attack</td><td width=\"9%\">Attacker</td>";
  rCndtn += "<td width=\"9%\">Attackee</td></tr>";

  return;
}

static QString clrSplt(const int iSn, const int iTn)
{
  short sR = 0xff;
  short sG = 0x00;
  short sB = 0x00;
  short sF = 0;

  QString tRGB;

  while(iTn-1>0)
  {
    sF = (short)(1.0*1020/(iTn-1));
    sF *= (iSn-1);

    if(!sF)
      break;

    sG +=sF;

    if(sG>0xff)
      sG = 0xff;

    if(sF-0xff>0)
      sR -= (sF-0xff);

    if(sR<0)
      sR = 0;

    if(sF-(2*0xff)>0)
      sB += sF-(2*0xff);

    if(sB>0xff)
      sB = 0xff;

    if(sF-(3*0xff)>0)
      sG -= sF-(3*0xff);
    break;
  }

  return tRGB.sprintf("%02x%02x%02x", sR, sG, sB);
}

void Analysis::buildReport(const void *pData)
{
  ATCKINFO *pAtckIf = (ATCKINFO *)pData;
  char *pEvtMsg = (char *)pData+ATCKIFLEN;
  QString tTblEnt;

  if(!pData)
    return;

  if(!pSId)
    pSId = new short[pAtckIf->uiAtckt];

  pSId[iRowsInTbl++] = pAtckIf->usSigId; 

  iSumAtckd += pAtckIf->uiAtckd;
  iSumAtckr += pAtckIf->uiAtckrs;
  iSumAtcks = pAtckIf->uiAtckt;

  if(strlen(pEvtMsg)>32)
  {
    char *pTmp = index(pEvtMsg+16, 32);

    if(!pTmp || (pTmp - pEvtMsg) > 32)
      *(pEvtMsg+32) = '\0';
  }
  
  QString tPrcntf;
  tPrcntf.sprintf("%4.1f%%", 100*(1.0*pAtckIf->uiAtcks/pAtckIf->uiAtckt));
  QString tPrcntb;
  tPrcntb.sprintf("%d%%",
                  100-(int)(100*(1.0*pAtckIf->uiAtcks/pAtckIf->uiAtckt)));

  tTblEnt = "<tr>";
  tTblEnt += "<td width=\"12%\"><small><a href=\"#";
  tTblEnt += QString("%1").arg(pAtckIf->usSigId)+"\">";
  tTblEnt += QString("%1").arg(pAtckIf->usSigId)+"</a>";
  tTblEnt += "</small></td><td width=\"28%\"><small>";
  tTblEnt += pEvtMsg;
  tTblEnt += "</small></td>";
  tTblEnt += "<td width=\"22%\"><table cellpadding=\"0\" cellspacing=\"0\" ";
  tTblEnt += "border=\"0\" width=\"100%\"><tbody><tr>";
  tTblEnt += "<td bgcolor=\"#"+clrSplt(iRowsInTbl, pAtckIf->uiTtlRows);
  tTblEnt += "\" width=\""+tPrcntf+"\" height=\"10\" style=\"border:0\">";
  tTblEnt += "&nbsp;</td><td bgcolor=\"white\" width=\""+tPrcntb+"\" ";
  tTblEnt += "height=\"10\" style=\"border:0\">&nbsp;</td>";
  tTblEnt += "</tr></tbody></table></td>";
  tTblEnt += "<td width=\"5%\"><small>"+tPrcntf+"</small></td>";

  if(pAtckIf->sPrrty==1)
  {
    tTblEnt += "<td width=\"8%\"><small><font color=\"#ff0000\">";
    tTblEnt += "High</font></small></td>";
  }
  else if(pAtckIf->sPrrty==2)
  {
    tTblEnt += "<td width=\"8%\"><small><font color=\"#cc33cc\">";
    tTblEnt += "Medium</font></small></td>";
  }
  else
  {
    tTblEnt += "<td width=\"8%\"><small><font color=\"#0000ff\">";
    tTblEnt += "Low</font></small></td>";
  }

  tTblEnt += "<td width=\"7%\"><small>"+QString("%1").arg(pAtckIf->uiAtcks);
  tTblEnt += "</small></td><td width=\"9%\"><small>";
  tTblEnt += QString("%1").arg(pAtckIf->uiAtckrs);
  tTblEnt += "</small></td><td width=\"9%\"><small>";
  tTblEnt += QString("%1").arg(pAtckIf->uiAtckd);
  tTblEnt += "</small></td></tr>";

  QTextStream tStrm(&tBuff);
  tStrm << tTblEnt;

  return;
}

static int reportData(const unsigned char *pData,
                      const unsigned short usLen,
                      const void *pObj)
{
  Analysis *pAnls = (Analysis *)pObj;
  ATCKINFO *pAtckInfo = (ATCKINFO *)pData;
  unsigned short usDataLen = 0;

  if(pData && pObj)
  {
    usDataLen = (unsigned short)ntohl(pAtckInfo->uiMsgLen)+ATCKIFLEN;

    if(usDataLen != usLen)
      return -1;

    pAnls->buildReport(pData);
    free(pAtckInfo);
    return 0;
  }

  return -1;
}

static int detailData(const unsigned char *pData,
                      const unsigned short usLen,
                      const void *pObj)
{
  Analysis *pAnls = (Analysis *)pObj;
  DTLINFO *pDtlIf = (DTLINFO *)pData;
  unsigned short usDataLen = 0;

  if(pData && pObj)
  {
    usDataLen = (unsigned short)ntohl(pDtlIf->uiMsgLen)+DTLIFLEN;

    if(usDataLen != usLen)
      return -1;

    pAnls->buildDtlData(pData);
    if(pAnls->pDtl)
      pAnls->pDtl->pPrgrs->setProgress(++pAnls->pDtl->iCntSteps);

    free(pDtlIf);
    return 0;
  }

  return -1;
}

void Analysis::genReport(void)
{
  char *pBrwsr = NULL;
  QString tSqlStmt;
  QString tCndtn;

  if(((MONITSRVR *)pMonit)->uiNoDbs)
  {
    QMessageBox::warning(this, "DataBase Status",
                               "Can Not Connect to DataBase");
    return;
  }

  if(genSqlStmt(tSqlStmt))
    return;

  pBrwsr = getCnfgCharElement("browser", ((MONITSRVR *)pMonit)->pSrvrCnfg);

  iRowsInTbl = 0;
  iSumAtcks = 0;
  iSumAtckr = 0;
  iSumAtckd = 0;
  genCondition(tCndtn);

  tBuff.setName("/tmp/report.html");
  tBuff.open(IO_ReadWrite|IO_Truncate);
  
  QTextStream tStrm(&tBuff);
  tStrm << RPTTBLHDR+tCndtn;

  dbFetchAtckInfo(pMonit, tSqlStmt.latin1(), reportData, this);

  tCndtn = "<tr><td width=\"12%\"><small>Total</small></td>";
  tCndtn += "<td width=\"28%\"></td><td width=\"22%\"></td>";
  tCndtn += "<td width=\"5%\"><small>100%</small></td><td width=\"8%\"></td>";
  tCndtn += "<td width=\"7%\"><small>"+QString("%1").arg(iSumAtcks);
  tCndtn += "</small></td><td width=\"9%\"><small>";
  tCndtn += QString("%1").arg(iSumAtckr);
  tCndtn += "</small></td><td width=\"9%\"><small>";
  tCndtn += QString("%1").arg(iSumAtckd);
  tCndtn += "</small></td></tr></tbody></table></body></html>";

  tStrm << tCndtn;

  int i = 0;

  for(i=0; i<iRowsInTbl; ++i)
  {
    char tTmp[8];
    QString tStmt = "select sensorId, eventTime, protocolID ,priority, "
                    "srcIp, srcPort, dstIp, dstPort, srcMac, dstMac, "
                    "eventMsg from events a, addrinfo b where "
                    "b.addrId = a.addrId ";
    tStmt += tFchCndtn+" signatureId = ";
    tStmt += QString("%1").arg(pSId[i])+" order by eventTime";

    sprintf(tTmp, "%d", pSId[i]);
    tCndtn = "<br><p id=";
    tCndtn += tTmp;
    tCndtn += ">Detail Information ";
    tCndtn += tTmp;
    tCndtn += "</p>";
    tStrm << tCndtn+DTLTBLHDR;
    dbFetchDetails(pMonit, tStmt.latin1(), detailData, this);
    tStrm << "</tbody></table></body></html>";
  }

  tBuff.close();
  delete pSId;
  pSId = NULL;

  if(!access(pBrwsr, R_OK|X_OK))
  {
    pid_t tPid;
    if((tPid = fork()) < 0)
    {
      QMessageBox::warning(this, "Startting Browser",
                                 "Cannot Start Browser process.");
    }
    else if(!tPid)
    {
      if(execlp(pBrwsr, rindex(pBrwsr, '/')+1,
                 "file:///tmp/report.html", (char *)0) == -1)
      {
        exit(127);
      }
    }
  }
  else
    QMessageBox::warning(this, "Browser", "Cannot Execute Browser!");

  return;
}

void Analysis::buildDtlData(const unsigned char *pData)
{
  DTLINFO *pDtlIf = (DTLINFO *)pData;
  char *pEvtMsg = (char *)pData+DTLIFLEN;
  struct tm *pDate = localtime((time_t *)&pDtlIf->ulEvtTime);
  QString tDtlData;
  QString tTmp;
  
  if(pDtl && !pDtl->iTtlSteps)
  {
    pDtl->iTtlSteps = pDtlIf->uiTtlRows;
    pDtl->pPrgrs->setTotalSteps(pDtl->iTtlSteps);
  }

  tDtlData = "<tr><td valign=\"top\"><small>"; 
  tDtlData += QString("%1").arg(pDtlIf->uiSnsrId);
  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 

  tTmp.sprintf("%d-%02d-%02d %02d:%02d:%02d",
                     1900+pDate->tm_year, pDate->tm_mon+1,
                     pDate->tm_mday, pDate->tm_hour,
                     pDate->tm_min, pDate->tm_sec);
  tDtlData += tTmp; 

  tDtlData += "</small><br></td>";
  tDtlData += "<td valign=\"top\"><small>"; 
  tDtlData += pEvtMsg;
  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 

  tTmp.sprintf("%ld.%ld.%ld.%ld", (pDtlIf->ulSrcIp>>24)&0xFF,
                                  (pDtlIf->ulSrcIp>>16)&0xFF,
                                  (pDtlIf->ulSrcIp>>8)&0xFF,
                                  pDtlIf->ulSrcIp&0xFF);
  tDtlData += tTmp; 

  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 

  if(pDtlIf->usPrtclId == 1)
    tDtlData += QString("%1(type)").arg(pDtlIf->usSrcPt&0xFF);
  else if(pDtlIf->usPrtclId == 6 || pDtlIf->usPrtclId == 17)
    tDtlData += QString("%1").arg(pDtlIf->usSrcPt);
  else
    tDtlData += "N/A";

  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 

  tTmp.sprintf("%ld.%ld.%ld.%ld", (pDtlIf->ulDstIp>>24)&0xFF,
                                  (pDtlIf->ulDstIp>>16)&0xFF,
                                  (pDtlIf->ulDstIp>>8)&0xFF,
                                  pDtlIf->ulDstIp&0xFF);
  tDtlData += tTmp; 

  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 

  if(pDtlIf->usPrtclId == 1)
    tDtlData += QString("%1(code)").arg(pDtlIf->usDstPt&0xFF);
  else if(pDtlIf->usPrtclId == 6 || pDtlIf->usPrtclId == 17)
    tDtlData += QString("%1").arg(pDtlIf->usDstPt);
  else
    tDtlData += "N/A";

  tDtlData += "<br></small></td>";

  tDtlData += "<td valign=\"top\"><small>"; 
  if(pDtlIf->sPrrty == 1)
    tDtlData += "<font color=\"#ff0000\">High</font>";
  else if(pDtlIf->sPrrty == 2)
    tDtlData += "<font color=\"#cc33cc\">Medium</font>";
  else
    tDtlData += "<font color=\"#0000ff\">Low</font>";
  tDtlData += "<br></small></td>";

  tDtlData += "<td valign=\"top\"><small>"; 
  if(pDtlIf->usPrtclId == 1)
    tDtlData += "ICMP";
  else if(pDtlIf->usPrtclId == 6)
    tDtlData += "TCP";
  else if(pDtlIf->usPrtclId == 17)
    tDtlData += "UDP";
  else if(pDtlIf->usPrtclId == 0)
    tDtlData += "IP";
  else
    tDtlData += QString("%1").arg(pDtlIf->usPrtclId);
  tDtlData += "<br></small></td>";

  tDtlData += "<td valign=\"top\"><small>"; 
  tDtlData += pDtlIf->cSrcMac;
  tDtlData += "<br></small></td>";
  tDtlData += "<td valign=\"top\"><small>"; 
  tDtlData += pDtlIf->cDstMac;
  tDtlData += "<br></small></td></tr>";

  QTextStream tStrm(&tBuff);
  tStrm << tDtlData + "\n";

  return;
}

static int referenceData(const unsigned char *pData,
                         const unsigned short usLen,
                         const void *pObj)
{
  Analysis *pAnls = (Analysis *)pObj;
  char *pRfnData = (char *)pData;

  if(pData && pObj)
  {
    char **pTmp = NULL;

    if(!pAnls->pRfnInfo)
    {
      pAnls->pRfnInfo = (char **)calloc((int)*pRfnData+1, sizeof(char *));
      pTmp = pAnls->pRfnInfo;
    }
    else
    {
      for(pTmp = pAnls->pRfnInfo; *pTmp; ++pTmp);
    }

    *pTmp = strdup(pRfnData+1);

    return 0;
  }

  return -1;
}

void Analysis::moreInfo(int iRow, int iClm, int iBut, const QPoint &rPstn)
{
  int iDetail = 0;
  unsigned long ulSigId = 0;
  bool bOk = false;

  if(iRow == -1 || iBut == 1)
    return;

  ulSigId = pRsltTbl->text(iRow, 0).toULong(&bOk);

  if(bOk == false)
    return;

  pInfoMenu = new QPopupMenu(this);
  iDetail = pInfoMenu->insertItem("Details");

  if(!((MONITSRVR *)pMonit)->uiNoDbs)
  {
    char **pTmp = pRfnInfo;

    if(ulSigId != ulPrevSid)
    {
      ulPrevSid = ulSigId;

      if(pRfnInfo) // Clean previous reference info.
      {
        while(*pTmp)
        {
          free(*pTmp);
          ++pTmp;
        }

        free(pRfnInfo);
        pRfnInfo = NULL;
      }

      QString tStmt;
      tStmt.sprintf("select httpReference from reference where "
                    "signatureId = %lu", ulSigId);

      dbFetchReference(pMonit, tStmt.latin1(), referenceData, this);
    }

    if(pRfnInfo)
    {
      pTmp = pRfnInfo;

      QPopupMenu *pRfnMenu = new QPopupMenu(this);
      connect(pRfnMenu, SIGNAL(activated(int)), this,
                        SLOT(execBrowser(int)));

      while(*pTmp)
      {
        pRfnMenu->insertItem(QPixmap((const char **)httprfn), *pTmp);
        ++pTmp;
      }

      pInfoMenu->insertSeparator();
      pInfoMenu->insertItem("Reference", pRfnMenu);
    }
  }

  int iId = pInfoMenu->exec(QCursor::pos());

  if(iId == iDetail)
  {
    genDetail(iRow, iClm, iBut, rPstn);
  }
}

void Analysis::execBrowser(int iIdx)
{
  char *pBrwsr = NULL;

  pBrwsr = getCnfgCharElement("browser", ((MONITSRVR *)pMonit)->pSrvrCnfg);

  if(!access(pBrwsr, R_OK|X_OK))
  {
    pid_t tPid;

    tPid = fork();

    if(!tPid)
    {
      if(execlp(pBrwsr, rindex(pBrwsr, '/')+1,
                pInfoMenu->text(iIdx).latin1(), (char *)0) == -1)
      {
        exit(127);
      }
    }

    return;
  }

  QMessageBox::warning(this, "Browser", "Cannot Execute Browser!");
}

void Analysis::genDetail(int iRow, int iClm, int iBut, const QPoint &rPstn)
{
  if((pDtl && pDtl->bWinOpened) || iRow == -1)
    return;

  pDtl = new DetailForm(&tBuff);
  pDtl->show();

  pDtl->bWinOpened = true;

  tEvtSigId = pRsltTbl->text(iRow, 0);
  QTimer *pTimer = new QTimer(this);
  pTimer->start(0, true);
  connect(pTimer, SIGNAL(timeout()), SLOT(genDtlFile()));

  return;
}

void Analysis::genDtlFile(void)
{
  QString tStmt = "select sensorId, eventTime, protocolID ,priority, "
                  "srcIp, srcPort, dstIp, dstPort, srcMac, dstMac, "
                  "eventMsg from events a, addrinfo b where "
                  "b.addrId = a.addrId ";
  tStmt += tFchCndtn+" signatureId = "; 
  tStmt += tEvtSigId+" order by eventTime"; 

  tBuff.setName("/tmp/detail.html");
  tBuff.open(IO_ReadWrite);
  QTextStream tStrm(&tBuff);
  dbFetchDetails(pMonit, tStmt.latin1(), detailData, this);
  tBuff.close();

  pDtl->openFile();
  pDtl->nextPage();

  return;
}

SetPriority::SetPriority(QTable *parent, EditType et,
                         const QString &rText, const short sPrrty)
:QTableItem(parent, et, rText)
{
  setText(rText);
  sPriority = sPrrty;
}

void SetPriority::paint(QPainter *pPaint, const QColorGroup &rCg,
                   const QRect &rCr, bool bSelected) 
{
  QColorGroup tCg(rCg);
  QColor tC = tCg.text();

  switch(sPriority)
  {
    case 1: /* High Priority */
                 tCg.setColor(QColorGroup::Text, Qt::red);
                 break;
    case 2: /* Mid Priority */
                 tCg.setColor(QColorGroup::Text, Qt::magenta);
                 break;
    default: /* Low Priority */
                 tCg.setColor(QColorGroup::Text, Qt::blue);
                 break;
  }

  QTableItem::paint(pPaint, tCg, rCr, bSelected);

  tCg.setColor(QColorGroup::Text, tC);

  return;
}

DetailForm::DetailForm(QFile *pFile, QWidget* parent, const char* name)
:QWidget(parent, name)
{
  iCntNext = 0;
  iCntSteps = 0;
  iTtlSteps = 0;
  lOffSet = NULL;
  bWinOpened = false;
  pDtlFile = pFile;

  if(!name)
     setName("DetailForm");
  resize(880, 280);
  setIcon(QPixmap(snort_xpm));
  setCaption(tr("Details"));
  pLayout = new QGridLayout(this, 1, 1, 6, 6);

  pPrgrs = new QProgressBar(this, "Progress Bar");
  pPrgrs->setProgress(0);
  pLayout->addWidget(pPrgrs, 1, 0);
  
  QSpacerItem* spacer = new QSpacerItem(20, 20, QSizePolicy::Expanding,
                                        QSizePolicy::Minimum);
  pLayout->addItem(spacer, 1, 1);

  pPrev = new QPushButton(this, "Prev");
  pPrev->setText(tr("Previous"));
  pPrev->setEnabled(false);
  pLayout->addWidget(pPrev, 1, 2);
  connect(pPrev, SIGNAL(clicked()), this, SLOT(prevPage()));

  pNext = new QPushButton(this, "Next");
  pNext->setText(tr("Next"));
  pNext->setEnabled(false);
  pLayout->addWidget(pNext, 1, 3);
  connect(pNext, SIGNAL(clicked()), this, SLOT(nextPage()));

  pClose = new QPushButton(this, "pClose");
  pClose->setText(tr("Close"));
  pClose->setDefault(true);
  pLayout->addWidget(pClose, 1, 4);
  connect(pClose, SIGNAL(clicked()), this, SLOT(close()));

  pTxtVw = new QTextView(this, "TextView");
  QBrush tBckPaper;
  tBckPaper.setColor(QColor(0xcc, 0xcc, 0xff));
  tBckPaper.setStyle(SolidPattern);
  pTxtVw->setPaper(tBckPaper);
  pLayout->addMultiCellWidget(pTxtVw, 0, 0, 0, 4);
}

void DetailForm::openFile(void)
{
  if(pDtlFile)
  {
    pDtlFile->open(IO_ReadOnly);
    lOffSet = new long[(int)(iTtlSteps/PAGESIZE)+1];
  }
}

void DetailForm::nextPage(void)
{
  QString tDataBlock;
  QTextStream tStrm(pDtlFile);

  lOffSet[iCntNext++] = pDtlFile->at();

  tDataBlock = "<html><body><div align=\"center\"><h1><small>";
  tDataBlock += "<b>Detail Information</b></small></h1></div><br>";
  tDataBlock += DTLTBLHDR;

  register int i = 0;

  while(i<PAGESIZE)
  {
    if(tStrm.atEnd())
      break;

    tDataBlock += tStrm.readLine();
    ++i;
  }

  if(tStrm.atEnd())
    iCntSteps = 0; 

  tDataBlock +="</tbody></table></body></html>";
  pTxtVw->setText(tDataBlock);

  if(iCntSteps > 0)
  {
    if(!pNext->isEnabled())
      pNext->setEnabled(true);
  }
  else
    pNext->setEnabled(false);

  if(!pPrev->isEnabled() && iCntNext > 1)
    pPrev->setEnabled(true);
}

void DetailForm::prevPage(void)
{
  QString tDataBlock;
  QTextStream tStrm(pDtlFile);

  --iCntNext;
  pDtlFile->at(lOffSet[iCntNext-1]);

  iCntSteps = iTtlSteps;

  tDataBlock = "<html><body><div align=\"center\"><h1><small>";
  tDataBlock += "<b>Detail Information</b></small></h1></div><br>";
  tDataBlock += DTLTBLHDR;

  register int i = 0;

  while(i<PAGESIZE)
  {
    tDataBlock += tStrm.readLine();
    ++i;
  }

  tDataBlock +="</tbody></table></body></html>";
  pTxtVw->setText(tDataBlock);

  if(iCntNext == 1)
    pPrev->setEnabled(false);
  else
  {
    if(!pPrev->isEnabled())
      pPrev->setEnabled(true);
  }

  if(!pNext->isEnabled())
    pNext->setEnabled(true);
}

void DetailForm::closeEvent(QCloseEvent *pEvt)
{
  pEvt->accept();
  bWinOpened = false;

  if(pDtlFile->isOpen())
  {
    pDtlFile->close();
    pDtlFile->remove();

    if(lOffSet)
      delete lOffSet;
  }

  return;
}

DetailForm::~DetailForm()
{
  if(pDtlFile->isOpen())
  {
    pDtlFile->close();
    pDtlFile->remove();

    if(lOffSet)
      delete lOffSet;
  }
}
