/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _ANALYSIS_H_
#define _ANALYSIS_H_

#include <qvariant.h>
#include <qwidget.h>
#include <qtable.h>
#include <qtextview.h>
#include <qevent.h>
#include <qfile.h>
#include <qpopupmenu.h>
#include <qprogressbar.h>
#include <qdatetimeedit.h>

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QComboBox;
class QLabel;
class QLineEdit;
class QPushButton;
class QString;

class SetPriority: public QTableItem
{
public:
  SetPriority(QTable *parent, EditType et,
              const QString &rText, const short);
  ~SetPriority(){}

  void paint(QPainter *, const QColorGroup &, const QRect &, bool);

private:
  short sPriority;
};

class DetailForm : public QWidget
{
  Q_OBJECT

public:
  DetailForm(QFile *pFile = 0, QWidget* parent = 0, const char* name = 0);
  ~DetailForm();

  QPushButton *pNext;
  QPushButton *pPrev;
  QPushButton *pClose;
  QTextView *pTxtVw;
  QProgressBar* pPrgrs;
  QFile *pDtlFile;

  int iCntNext;
  int iCntSteps;
  int iTtlSteps;
  long *lOffSet;
  bool bWinOpened;

  void openFile(void);

public slots:
  void nextPage(void);
  void prevPage(void);

protected:
  QGridLayout* pLayout;

  void closeEvent(QCloseEvent *);
};

class Analysis : public QWidget
{ 
  Q_OBJECT

public:
  Analysis(void *pUser = 0, QWidget* parent = 0, const char* name = 0);
  ~Analysis()
  {
    if(pDtl)
      delete pDtl;
  }

  QLabel* snsrId;
  QLineEdit* pSnsrId;
  QLabel* sigId;
  QLineEdit* pSigId;
  QLabel* source;
  QLabel* srcIpFrom;
  QLineEdit* pSrcIpFrom;
  QLabel* srcIpTo;
  QLineEdit* pSrcIpTo;
  QLabel* srcPtFrom;
  QLineEdit* pSrcPtFrom;
  QLabel* srcPtTo;
  QLineEdit* pSrcPtTo;
  QLabel* destination;
  QLabel* dstIpFrom;
  QLineEdit* pDstIpFrom;
  QLabel* dstIpTo;
  QLineEdit* pDstIpTo;
  QLabel* dstPtFrom;
  QLineEdit* pDstPtFrom;
  QLabel* dstPtTo;
  QLineEdit* pDstPtTo;
  QLabel* startDT;
  QDateTimeEdit *pStartDT;
  QLabel* endDT;
  QDateTimeEdit* pEndDT;
  QLabel* protocol;
  QComboBox* pProtocol;
  QLabel* priority;
  QComboBox* pPriority;
  QPushButton* pAnalysis;
  QPushButton* pReport;
  QTable* pRsltTbl;
  QPopupMenu *pInfoMenu;

  int iSumAtcks;
  int iSumAtckr;
  int iSumAtckd;
  int iRowsInTbl;
  QString tEvtSigId;
  QString tFchCndtn;
  QFile tBuff;
  DetailForm *pDtl;
  short *pSId;
  char **pRfnInfo;
  unsigned long ulPrevSid;

  int genSqlStmt(QString &);
  void genCondition(QString &);
  void buildAnalysis(const void *);
  void buildReport(const void *);
  void buildDtlData(const unsigned char *);

public slots:
  void genAnalysis(void);
  void genReport(void);
  void execBrowser(int);
  void moreInfo(int, int, int, const QPoint &);
  void genDetail(int, int, int, const QPoint &);
  void genDtlFile(void);

protected:
  QGridLayout* pLayout;
  void *pMonit;
};

#endif
