/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <netinet/in.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qtimer.h>

#include <gmlmonit.h>
#include <snortdata.h>

#include <statsview.h>
#include <snsrup.xpm>
#include <snsrsp.xpm>
#include <snsrdn.xpm>
#include <snort16.xpm>
#include <marble.xpm>
#include <stop.xpm>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

StatsView::StatsView(void *pUser, QWidget *parent, const char *name) :
QWidget(parent, name)
{
  pMonit = pUser;

  if(parent)
    parent->installEventFilter(this);

  tSList.setAutoDelete(TRUE);

  tGrnIcon = QPixmap(snsrup_xpm); 
  tRedIcon = QPixmap(snsrdn_xpm); 

  QHBoxLayout *pHLay = new QHBoxLayout(this);
  pHLay->setMargin(3);

  QVBoxLayout *pVLay = new QVBoxLayout(pHLay);
  pVLay->setMargin(3);

  pIcon = new QIconView(this);
  pIcon->setSelectionMode(QIconView::Extended);
  pVLay->addWidget(pIcon);

  pIcnLabel = new QLabel("Sensor Connected: 0", this);
  pVLay->addWidget(pIcnLabel);

  connect(pIcon, SIGNAL(rightButtonPressed(QIconViewItem *, const QPoint &)),
                 this, SLOT(processRequest(QIconViewItem *, const QPoint &)));

  QTimer *pTimer = new QTimer(this);
  pTimer->start(2000);
  connect(pTimer, SIGNAL(timeout()), SLOT(buildSnsrIcon()));
}

QIconViewItem *StatsView::searchView(const QIconView *pView,
                                     const QString &rStr)
{
  QIconViewItem *pIcon = pView->firstItem();

  while(pIcon)
  {
    if((pIcon->text()).compare(rStr) == 0)
      return pIcon;

    if(pIcon == pView->lastItem()) /* Only one item in the view */
      return NULL;
    else
      pIcon = pIcon->nextItem();
  }

  return NULL;
}

bool StatsView::eventFilter(QObject *pObj, QEvent *pEvt)
{
  if(pEvt->type() == 1001) /* User defined */
    return true;

  return QWidget::eventFilter(pObj, pEvt);
}

bool StatsView::event(QEvent *pEvt)
{
  if(pEvt->type() == 1001) /* User defined */
  {
    flareSnsrIcon();
    return true;
  }

  return QObject::event(pEvt);
}

void StatsView::insertSttData(void *pSnsrStats)
{
  SNSRSTATS *pSts = (SNSRSTATS *)pSnsrStats;

  if(!pSts)
    return;

  StatsData *pStatItem = NULL;

  for(pStatItem=tSList.first(); pStatItem; pStatItem=tSList.next())
  {
    if(pStatItem->getSnsrId() == (ntohl(pSts->ulSensorId)))
    {
      pStatItem->ulAltPkts = pSts->ulAltPkts;
      pStatItem->ulLogPkts = pSts->ulLogPkts;
      pStatItem->ulPsdPkts = pSts->ulPsdPkts;
      pStatItem->ulTcpPkts = pSts->ulTcpPkts;
      pStatItem->ulUdpPkts = pSts->ulUdpPkts;
      pStatItem->ulIcmpPkts = pSts->ulIcmpPkts;
      pStatItem->ulArpPkts = pSts->ulArpPkts;
      pStatItem->ulIpxPkts = pSts->ulIpxPkts;
      pStatItem->ulOthPkts = pSts->ulOthPkts;
      pStatItem->ulDscdPkts = pSts->ulDscdPkts;
      pStatItem->ulFrgIPPkts = pSts->ulFrgIPPkts;
      pStatItem->ulRbtIpPkts = pSts->ulRbtIpPkts;
      pStatItem->ulStrmTcpPkts = pSts->ulStrmTcpPkts;
      pStatItem->ulRcnsTcpPkts = pSts->ulRcnsTcpPkts;
      pStatItem->ulRcnsStrmPkts = pSts->ulRcnsStrmPkts;
      pStatItem->sensorTime.tv_sec = pSts->sensorTime.tv_sec;
      pStatItem->sensorTime.tv_usec = pSts->sensorTime.tv_usec;

      pStatItem->setUpdateFlag(true);
      QApplication::sendEvent(this, new QEvent((QEvent::Type)1001));
    }
  }
}

void *StatsView::getSnsrInfo(unsigned long ulSId)
{
  MONITSRVR *pMs = (MONITSRVR *)pMonit;
  GDATAPOLL *pInfo = NULL;
  CLIENTINF *pCrnt = NULL;
  
#define CLNTINFO pInfo->pNext->pAny
  gmlMxLock(pMs->pIfMtx);
  GETFIRST(pMs->pClntInfo, pInfo);
  while(pInfo->pNext && CLNTINFO)
  {
    if(((CLIENTINF *)CLNTINFO)->ulClntId == ulSId)
    {
      pCrnt = (CLIENTINF *)CLNTINFO;
      break;
    }

    pInfo=pInfo->pNext;
  }
  gmlMxUnLock(pMs->pIfMtx);

  return (void *)pCrnt;
}

void StatsView::buildSnsrIcon(void)
{
  if(!pMonit)
    return;

  MONITSRVR *pMs = (MONITSRVR *)pMonit;
  GDATAPOLL *pInfo = NULL;

  gmlMxLock(pMs->pIfMtx);
  GETFIRST(pMs->pClntInfo, pInfo);
  
  unsigned short usCntClnt = 0;

  while(pInfo->pNext && CLNTINFO)
  {
    ++usCntClnt;

    CLIENTINF *pCrnt = (CLIENTINF *)CLNTINFO;
#undef CLNTINFO

    QString strIPAddr = QString("%1.%2.%3.%4").arg((pCrnt->ulRmtIP>>24)&0xFF).
                                               arg((pCrnt->ulRmtIP>>16)&0xFF).
                                               arg((pCrnt->ulRmtIP>>8)&0xFF).
                                               arg(pCrnt->ulRmtIP&0xFF);

    QIconViewItem *pViewItem = NULL; 
    StatsData *pNewData = NULL;

    if(!(pViewItem = searchView(pIcon, strIPAddr)))
    {
      pViewItem = new QIconViewItem(pIcon, strIPAddr, tGrnIcon);
      pNewData = new StatsData(pCrnt->ulClntId);
      pNewData->setIconItem(pViewItem);
      tSList.append(pNewData);
    }
    else /* Update the IconView */
    {
      StatsData *pStatItem = NULL;
      
      for(pStatItem=tSList.first(); pStatItem; pStatItem=tSList.next())
      {
        if(pStatItem->getIconItem() == pViewItem)
        {
          if(pStatItem->usFlipIcon == 0)
            pViewItem->setPixmap(tGrnIcon, false, true);
          else if(pStatItem->usFlipIcon == SENSORUP)
            pStatItem->usFlipIcon = 0;
        }
      }
    }

    pInfo=pInfo->pNext;
  }
  gmlMxUnLock(pMs->pIfMtx);

  if(tSList.count() > usCntClnt)
  {
    StatsData *pStatItem = NULL;

    for(pStatItem=tSList.first(); pStatItem; pStatItem=tSList.next())
    {
      if(usCntClnt == 0)
      {
        delete pStatItem->getIconItem();
        tSList.remove();
      }
      else
      {
	if(!getSnsrInfo(pStatItem->getSnsrId()))
        {
          --usCntClnt;
          delete pStatItem->getIconItem();
          tSList.remove();
        }
      }
    } /* for */
  }

  pIcnLabel->setText(QString("Sensor Connected: %1").arg(usCntClnt));
}

void StatsView::processRequest(QIconViewItem *pItem, const QPoint &rIcon)
{
  if(pItem)
  {
    StatsData *pStatItem = NULL;
    int iStatistic = 0;
    int iSuspendIt = 0;
    int iResumesIt = 0;
    int iStopItNow = 0;

    QPopupMenu *pMenu = new QPopupMenu(pIcon);
    iStatistic = pMenu->insertItem(QString("Statistic"));
    iSuspendIt = pMenu->insertItem(QString("Suspend Sensor"));
    iResumesIt = pMenu->insertItem(QString("Resume Sensor"));
    iStopItNow = pMenu->insertItem(QString("Stop Sensor"));

    int iId = pMenu->exec(rIcon);

    if(iId != -1)
    {
      for(pStatItem=tSList.first(); pStatItem; pStatItem=tSList.next())
      {
        if(pStatItem->getIconItem() == pItem)
          break;
      }
    }
    else
      return;

    if(pStatItem->usFlipIcon == SNSRSTOP)
      return;

    if(iId == iStatistic)
    {
      if(!pStatItem->getStatsPops())
        pStatItem->setStatsPops(new StatsPops());

      pStatItem->getStatsPops()->popupWindow(pStatItem);
    }
    else if(iId == iSuspendIt)
    {
      pStatItem->usFlipIcon = SNSRSUSP;
      ((CLIENTINF *)getSnsrInfo(pStatItem->getSnsrId()))->ucStatus = SNSRSUSP;
      pItem->setPixmap(QPixmap(snsrsp_xpm));
    }
    else if(iId == iResumesIt)
    {
      pStatItem->usFlipIcon = SENSORUP;
      ((CLIENTINF *)getSnsrInfo(pStatItem->getSnsrId()))->ucStatus = SENSORUP;
      pItem->setPixmap(QPixmap(snsrdn_xpm));
    }
    else if(iId == iStopItNow)
    {
      pStatItem->usFlipIcon = SNSRSTOP;
      ((CLIENTINF *)getSnsrInfo(pStatItem->getSnsrId()))->ucStatus = SNSRSTOP;
      pItem->setPixmap(QPixmap(stop));
    }
  }
}

void StatsView::flareSnsrIcon()
{
  StatsData *pStatItem = NULL;
      
  for(pStatItem=tSList.first(); pStatItem; pStatItem=tSList.next())
  {
    if(pStatItem->getUpdateFlag() == true)
    {
      if(pStatItem->getIconItem())
      {
        if(pStatItem->usFlipIcon == 0)
        {
          pStatItem->getIconItem()->setPixmap(tRedIcon, false, true);
          pStatItem->usFlipIcon = SENSORUP;
        }

        if(pStatItem->getStatsPops())
          pStatItem->getStatsPops()->updateDisplay(pStatItem);
      }

      pStatItem->setUpdateFlag(false);
      break;
    }
  }
}

StatsPops::StatsPops() : QVBox(0, 0)
{
  bWinOpened = false;

  setIcon(QPixmap(snort_xpm));
  setCaption(QString("Monitor Version: %1").arg(VERSION));

  pView = new QTextView(this);
  pView->setMinimumSize(256, 480);

  QBrush tBckPaper;
  tBckPaper.setPixmap(QPixmap(marble_xpm));
  pView->setPaper(tBckPaper);

  pButton = new QHBox(this);
  pButton->setMargin(8);

  pClose = new QPushButton("&Close", pButton);

  connect(pClose, SIGNAL(clicked()), this, SLOT(close()));
}

void StatsPops::updateDisplay(StatsData *pData)
{
  if(!pData || !bWinOpened || !pView)
    return;

  QString strStats;
  struct tm *pTime = localtime(&(pData->sensorTime.tv_sec));

  strStats.sprintf("<center><big><b>Sensor ID:</b> %lu</big>"
                   "<br><hr></center>"
                   "<b>Sensor Time:</b> %d-%02d-%02d %02d:%02d:%02d<br>"
                   "<center><b>Action Statistics</b></center>"
                   "<b>ALERTS:</b> %lu<br>"
                   "<b>LOGGED:</b> %lu<br>"
                   "<b>PASSED:</b> %lu<br><br>"
                   "<center><b>Breakdown by protocols</b></center>"
                   "<b>TCP:</b> %lu<br>"
                   "<b>UDP:</b> %lu<br>"
                   "<b>ICMP:</b> %lu<br>"
                   "<b>ARP:</b> %lu<br>"
                   "<b>IPX:</b> %lu<br>"
                   "<b>OTHER:</b> %lu<br>"
                   "<b>DISCARD:</b> %lu<br><br>"
                   "<center><b>Fragmentation Statistics</b></center>"
                   "<b>Fragmented IP Packets:</b> %lu<br>"
                   "<b>Rebuilt IP Packets:</b> %lu<br><br>"
                   "<center><b>TCP Stream Reassembly</b></center>"
                   "<b>TCP Packets Used:</b> %lu<br>"
                   "<b>Stream Trackers:</b> %lu<br>"
                   "<b>Stream flushes:</b> %lu<br>",
                   pData->getSnsrId(),
                   1900+pTime->tm_year,
                   pTime->tm_mon+1,
                   pTime->tm_mday,
                   pTime->tm_hour,
                   pTime->tm_min,
                   pTime->tm_sec,
                   pData->ulAltPkts,
                   pData->ulLogPkts,
                   pData->ulPsdPkts,
                   pData->ulTcpPkts,
                   pData->ulUdpPkts,
                   pData->ulIcmpPkts,
                   pData->ulArpPkts,
                   pData->ulIpxPkts,
                   pData->ulOthPkts,
                   pData->ulDscdPkts,
                   pData->ulFrgIPPkts,
                   pData->ulRbtIpPkts,
                   pData->ulStrmTcpPkts,
                   pData->ulRcnsTcpPkts,
                   pData->ulRcnsStrmPkts);

  pView->setText(strStats);
}

void StatsPops::popupWindow(StatsData *pData)
{
  if(bWinOpened)
    return;

  QString strInit;
  strInit.sprintf("<center><big><b>Sensor ID:</b> %lu</big>"
                  "<br><hr><br><b>Waiting Data ...</b></center>",
                   pData->getSnsrId());

  pView->setText(strInit);

  show();
  bWinOpened = true;
}

void StatsPops::closeEvent(QCloseEvent *pEvt)
{
  pEvt->accept();
  bWinOpened = false;
}
