/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _STATISVIEW_H_
#define _STATISVIEW_H_

#include <stdio.h>
#include <sys/time.h>

#include <qwidget.h>
#include <qpixmap.h>
#include <qiconview.h>
#include <qpushbutton.h>
#include <qtextview.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qevent.h>
#include <qlist.h>

class StatsData;

class StatsPops : public QVBox
{
  Q_OBJECT

public:
  StatsPops();
  ~StatsPops()
  {
    if(pClose)
      delete pClose;

    if(pButton)
      delete pButton;

    if(pView)
      delete pView;
  }

  void updateDisplay(StatsData *);
  void popupWindow(StatsData *);

public slots:
  void closeEvent(QCloseEvent *);

private:
  QTextView *pView;
  QHBox *pButton;
  QPushButton *pClose;
  bool bWinOpened;
};

class StatsData
{
public:
  StatsData(unsigned long ulSId)
  {
    ulSensorId = ulSId;
    pIconLnk = NULL;
    pStatsPop = NULL;
    bUpdated = false;
    usFlipIcon = 0;
  }

  ~StatsData()
  {
    if(pStatsPop)
      delete pStatsPop;
  }

  unsigned long getSnsrId(void)
  {
    return ulSensorId;
  }

  void setIconItem(QIconViewItem *pItem)
  {
    pIconLnk = pItem;
  }

  QIconViewItem *getIconItem(void)
  {
    return pIconLnk;
  }

  void setStatsPops(StatsPops *pPops)
  {
    pStatsPop = pPops;
  }

  StatsPops *getStatsPops(void)
  {
    return pStatsPop;
  }

  void setUpdateFlag(bool bFlag)
  {
    bUpdated = bFlag;
  }

  bool getUpdateFlag(void)
  {
    return bUpdated;
  }

  unsigned long ulAltPkts;
  unsigned long ulLogPkts;
  unsigned long ulPsdPkts;
  unsigned long ulTcpPkts;
  unsigned long ulUdpPkts;
  unsigned long ulIcmpPkts;
  unsigned long ulArpPkts;
  unsigned long ulIpxPkts;
  unsigned long ulOthPkts;
  unsigned long ulDscdPkts;
  unsigned long ulFrgIPPkts;
  unsigned long ulRbtIpPkts;
  unsigned long ulStrmTcpPkts;
  unsigned long ulRcnsTcpPkts;
  unsigned long ulRcnsStrmPkts;
  struct timeval sensorTime;

  unsigned short usFlipIcon;

private:
  unsigned long ulSensorId;
  QIconViewItem *pIconLnk;
  StatsPops *pStatsPop;
  bool bUpdated;
};

class StatsView : public QWidget
{
  Q_OBJECT

public:
  StatsView(void *pUser = 0, QWidget *parent = 0, const char *name = 0);
  ~StatsView()
  {
    if(pIcon)
      delete pIcon;
  }

  bool eventFilter(QObject *, QEvent *);
  bool event(QEvent *);

  void insertSttData(void *);
  void flareSnsrIcon();

public slots:
  void buildSnsrIcon();
  void processRequest(QIconViewItem *, const QPoint &);

protected:
  QList<StatsData> tSList;

  QIconViewItem *searchView(const QIconView *, const QString &);
  void *getSnsrInfo(unsigned long);

private:
  void *pMonit;
  QIconView *pIcon;
  QLabel *pIcnLabel;
  QPixmap tGrnIcon, tRedIcon;
};

#endif
