/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <openssl/x509.h>
#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>

#include <printmsg.h>

void *readCert(const char *pFileName)
{
  FILE *pfd = NULL;
  X509 *pCert = NULL;

  if(!pFileName)
    return NULL;

  if(!(pfd = fopen(pFileName, "r")))
  {
    errorMessage("%s\n", strerror(errno));
    return NULL;
  }

  if(!(pCert = PEM_read_X509(pfd, NULL, NULL, NULL)))
    errorMessage("%s\n", ERR_reason_error_string(ERR_get_error()));

  fclose(pfd);
  return (void *)pCert;
}

void *readPrivKey(const char *pFileName)
{
  FILE *pfd = NULL;
  EVP_PKEY *pPrivKey = NULL;

  if(!pFileName)
    return NULL;

  if(!(pfd = fopen(pFileName, "r")))
  {
    errorMessage("%s\n", strerror(errno));
    return NULL;
  }

  if(!(pPrivKey = PEM_read_PrivateKey(pfd, NULL, NULL, NULL)))
    errorMessage("%s\n", ERR_reason_error_string(ERR_get_error()));

  fclose(pfd);
  return (void *)pPrivKey;
}
