/*
 * Copyright (C) 2001, 2002 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <glist.h>

inline GnrcList *initGList(void)
{
  return((GnrcList *)calloc(1, sizeof(GnrcList)));
}

inline GnrcList *addElm(GnrcList *pList)
{
   GnrcList *pLast = NULL;

  if(!pList)
    return NULL;

  GETLAST(pList, pLast);

  /* Allocte a new one */
  if(!(pLast->pNext = (GnrcList *)calloc(1, sizeof(GnrcList))))
  {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
    errorMessage("%s\n", strerror(errno));
#else
    errorMessage("calloc failed.\n");
#endif

    return NULL;
  }

  pLast->pNext->pPrev = pLast;

  return pLast->pNext;
}

inline GnrcList *getElm(GnrcList *pList, void *pValue, pCmpFnc cmpFnc)
{
  GnrcList *pFirst = NULL;

  if(!pList || !pValue)
    return NULL;

  GETFIRST(pList, pFirst);

  while(pFirst->pNext)
  {
    if(cmpFnc)
    {
      if(!(*cmpFnc)(pFirst->pNext->pAny, pValue))
        return pFirst->pNext;
    }
    else
    {
      if(pFirst->pNext == (GnrcList *)pValue)
        return pFirst->pNext;
    }
 
    pFirst = pFirst->pNext;
  }

  return NULL;
}

inline void delElm(GnrcList *pList)
{
  if(!pList)
    return;

  if(pList->pPrev)
    pList->pPrev->pNext = pList->pNext;

  if(pList->pNext)
    pList->pNext->pPrev = pList->pPrev;

  if(pList->pAny)
  {
    if(pList->pfAnyFree)
      (*pList->pfAnyFree)(pList->pAny);

    free(pList->pAny);
  }

  free(pList);
}

inline void delAllElm(GnrcList *pList)
{
  GnrcList *pFirst = NULL;

  if(!pList)
    return;

  GETFIRST(pList, pFirst);

  /* Skip the root node */
  if(pFirst && pFirst->pNext)
    pFirst = pFirst->pNext;

  while(pFirst->pNext)
  {
    pFirst = pFirst->pNext;
    delElm(pFirst->pPrev);
  }

  delElm(pFirst);
}
