/*
 *  $Id$
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <printmsg.h>
#include <sqlodbc.h>
#include <glist.h>

typedef struct RfnData
{
  char *pRfnSys;
  char *pRfnUrl;
} RFNDATA;

/* ODBD stuff */
void *pHEnv = NULL;
void *pHDbc = NULL;
/* Reference list */
GnrcList *pGRfn = NULL;

static inline void trimSpace(char *pStr)
{
  char *pTmp = pStr;
  int iC = 0;

  while((iC=*pStr++))
  {
    if(!isspace(iC))
      *pTmp++ = (char)iC;
  }

  *pTmp++ = '\0';
}

static inline void freeRfnData(void *pRfn)
{
  if(pRfn)
  {
    free(((RFNDATA *)pRfn)->pRfnSys);
    free(((RFNDATA *)pRfn)->pRfnUrl);
  }
}

static int buildRfnList(char *pFileName)
{
  FILE *pF = NULL;
  char tBuff[256];
  char *pSys = NULL;
  char *pUrl = NULL;
  int iErr = 0;

  if(!pFileName || !pGRfn)
    return -1; 

  pF = fopen(pFileName, "r");

  if(!pF)
  {
    errorMessage("%s: %s\n", strerror(errno), pFileName);
    return -1;
  }

  while(fgets(tBuff, 256, pF))
  {
    if(tBuff[0] != '#' && strlen(tBuff) > 2)
    {
      GnrcList *pTmp = NULL;
      RFNDATA *pRfn = NULL;

      if(!(pRfn = (RFNDATA *)calloc(1, sizeof(RFNDATA))))
      {
        iErr = 1;
        break;
      }

      trimSpace(tBuff);

      pSys = index(tBuff, ':')+1;
      pUrl = strstr(tBuff, "http");

      pRfn->pRfnUrl = strdup(pUrl);
      pSys[pUrl - pSys] = '\0';
      pRfn->pRfnSys = strdup(pSys);

      pTmp = addElm(pGRfn);
      pTmp->pfAnyFree = freeRfnData;
      pTmp->pAny = pRfn;
    } // if
  } // for

  fclose(pF);

  if(iErr)
    return -1;

  return 0;
}

static int rfnCmpr(void *pRfn, void *pValue)
{
  RFNDATA *pRFNDt = pRfn;
  
  return(strcasecmp(pRFNDt->pRfnSys, (char *)pValue));
}

static int insertDb(char *pRfn, const long lSid)
{
  char tStmt[256];
  char *pRfnSys = NULL;
  char *pRfnId = NULL;
  GnrcList *pTmpRfn = NULL;

  if(!pRfn || !lSid)
    return -1;

  pRfnSys = pRfn;
  pRfnId = index(pRfn, ',');

  if(!pRfnId)
  {
    errorMessage("Incorrect format!\n");
    return -1;
  }

  *pRfnId++ = 0;

  trimSpace(pRfnSys);
  trimSpace(pRfnId);

  pTmpRfn = getElm(pGRfn, pRfnSys, rfnCmpr);

  if(pTmpRfn)
  {
    SQLMSG tSqlMsg;

    memset(tStmt, 0, sizeof(tStmt));
    sprintf(tStmt, "insert into reference values (%ld, '%s%s')",
                    lSid, ((RFNDATA *)(pTmpRfn->pAny))->pRfnUrl, pRfnId);

    printf("%s\n", tStmt);

    if(dirExec(pHEnv, pHDbc, tStmt, &tSqlMsg))
    {
      errorMessage("dirExec failed ... %ld %s\n",
                                        tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
      if(tSqlMsg.sqlcd == 1062) /* Duplicated row */
      {
        printf("Ignore duplicate entry!\n");
      }
      else
        return -1;
    }
  }

  return 0;
}

static int parseRfn(FILE *pFd)
{
  char tBuff[1024];
  char *pPrev = NULL;

  if(!pFd)
    return -1;

  memset(tBuff, 0, sizeof(tBuff));

  while(fgets((char *)&tBuff, 1024, pFd))
  {
    char *pNewLine = NULL;
    int iErr = 0;

    if(*tBuff != '#' && *tBuff != ';' && *tBuff != 0x0a && strlen(tBuff) > 2)
    {
      char *pCrnt = tBuff;
      char *pTmp = NULL;

      if(pPrev)
      {
        pNewLine = (char *)calloc(1, strlen(pCrnt) + strlen(pPrev) + 1);

        if(!pNewLine)
        {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
          errorMessage("%s\n", strerror(errno));
#else
          errorMessage("calloc failed.\n");
#endif
          return -1;
        }

	strncat(pNewLine, pPrev, strlen(pPrev)); 
	strncat(pNewLine, pCrnt, strlen(pCrnt)); 
        free(pPrev);
        pPrev = NULL;
        pCrnt = pNewLine;
      }
      
      if((pTmp = rindex(pCrnt, '\\')))
      {
        *pTmp = 0;
        pPrev = strdup(pCrnt);
      }
      else
      {
        char *pRfn = NULL;
        char *pSid = NULL;
        long lSid = 0;
        char tRfn[256];

        if((pRfn = strstr(pCrnt, "reference:")))
        {
          /* Get sid */
          if((pSid = strstr(pCrnt, "sid:")))
          {
            pSid = pSid+4; /* Skip "sid:" */

            while(isspace((int)*pSid))
              pSid++;

            sscanf(pSid, "%ld;*", &lSid);
          }

          if(!lSid)
          {
            errorMessage("Incorrect format!\n");
            break;
          }

          do
          {
            pRfn = pRfn+10; /* Skip "reference:" */
          
            while(isspace((int)*pRfn))
              pRfn++;

            memset(tRfn, 0, sizeof(tRfn));
            sscanf(pRfn, "%[^;];*", tRfn);

            if(!strlen(tRfn))
            {
              errorMessage("Incorrect format!\n");
              break;
            }

            if((iErr = insertDb(tRfn, lSid)))
              break;
            
          } while((pRfn = strstr(pRfn, "reference:")));
        }
      }

      if(pNewLine)
        free(pNewLine);
    }

    if(iErr)
      return -1;

    memset(tBuff, 0, sizeof(tBuff));
  } /* while */

  return 0;
}

static int connectToDb(char *pDbName, char *pDbUser, char *pPsWord)
{
  SQLMSG tSqlMsg;

  if(dbInitEnv(&pHEnv, &tSqlMsg))
  {
    errorMessage("dbInitDbExit failed ... %ld %s\n",
                                  tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
    return -1;
  }

  if(dbConnect(pDbName, pDbUser, pPsWord, pHEnv, &pHDbc, 0, &tSqlMsg))
  {
    errorMessage("dbConnect failed ... %ld %s\n",
                                  tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
    return -1;
  }

  return 0;
}

static void terminateDb(void)
{
  SQLMSG tSqlMsg;

  if(dbDisconn(pHEnv, pHDbc, &tSqlMsg))
    errorMessage("dbDisconn failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);

  if(dbExitEnv(pHEnv, &tSqlMsg))
    errorMessage("dbEixtEnv failed ... %ld %s\n",
                                tSqlMsg.sqlcd, tSqlMsg.pErrMsg);
}

int main(int argc, char **argv)
{
  FILE *pFd = NULL;
  int i = 0;
  int iC = 0;
  char *pRfnFile = NULL;
  char *pDbName = NULL;
  char *pUsName = NULL;
  char *pPsWord = NULL;
  int iErr = 0;

  while((iC = getopt(argc, argv, "r:d:u:p:")) != -1)
  {
    switch(iC)
    {
      case 'r':
                pRfnFile = optarg;
                break;
      case 'd':
                pDbName = optarg; 
                break;
      case 'u':
                pUsName = optarg;
                break;
      case 'p':
                pPsWord = optarg;
                break;
    }
  }

  if(iC == -1 && argc < 10)
  {
    printf("Usage: %s -r reference.config -d dbname -u dbuser -p dbpasswd"
           " rule_files ...\n", argv[0]);

    return -1;
  }
  else
  {
    pGRfn = initGList();
  
    if(buildRfnList(pRfnFile))
      iErr = 1;
    else
    {
      if(connectToDb(pDbName, pUsName, pPsWord))
        iErr = 1;
    }
      
    if(!iErr)
    {
      for(i=optind; i<argc; ++i)
      {
        pFd = fopen(argv[i], "r");

        if(!pFd)
        {
          errorMessage("%s: %s\n", strerror(errno), argv[i]);
          iErr = 1;
          break;
        }

        iErr = parseRfn(pFd);
        fclose(pFd);

        if(iErr)
          break;
      }
    }

    if(pHEnv && pHDbc)
      terminateDb();

    delAllElm(pGRfn);

    if(iErr)
      return -1;
  }

  return 0;
}
