/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  $Id$
 */

#include <string.h>
#include <sqlodbc.h>

int dbInitEnv(void **pHEnv, SQLMSG *pSqlMsg)
{
  if(SQLAllocHandle(SQL_HANDLE_ENV, NULL, pHEnv))
  {
    if(pSqlMsg)
      dbErrMsg(*pHEnv, NULL, NULL, pSqlMsg);
    return -1;
  }

  SQLSetEnvAttr(*pHEnv, SQL_ATTR_ODBC_VERSION,
               (SQLPOINTER)SQL_OV_ODBC3, SQL_IS_UINTEGER);
  return 0;
}

int dbExitEnv(void *pHEnv, SQLMSG *pSqlMsg)
{
  if(!pHEnv)
    return -1;

  if(SQLFreeHandle(SQL_HANDLE_ENV, pHEnv))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, NULL, NULL, pSqlMsg);
    return -1;
  }

  return 0;
}

int dbConnect(char *pDbSvName, char *pUserId, char *pPasswd,
              void *pHEnv, void **pHDbc, unsigned long ulCnOpt,
              SQLMSG *pSqlMsg)
{ 
  int iRc = 0;

  if(!pHEnv || !pDbSvName || !pUserId)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(SQLAllocHandle (SQL_HANDLE_DBC, pHEnv, pHDbc))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, *pHDbc, NULL, pSqlMsg);
    return -1;
  }

  if(SQLSetConnectOption(*pHDbc, SQL_AUTOCOMMIT, ulCnOpt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, *pHDbc, NULL, pSqlMsg);
    return -1;
  }

  iRc = SQLConnect(*pHDbc, pDbSvName, SQL_NTS, pUserId, SQL_NTS,
                  pPasswd, SQL_NTS);

  if(iRc != SQL_SUCCESS && iRc != SQL_SUCCESS_WITH_INFO)
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, *pHDbc, NULL, pSqlMsg);
    return -1;
  }

  return 0;
}

int dbDisconn(void *pHEnv, void *pHDbc, SQLMSG *pSqlMsg)
{
  if(!pHEnv || !pHDbc )
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(SQLEndTran(SQL_HANDLE_DBC, pHDbc, SQL_COMMIT))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, NULL, pSqlMsg);
  }

  if(SQLDisconnect(pHDbc))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, NULL, pSqlMsg);
  }

  if(SQLFreeHandle(SQL_HANDLE_DBC, pHDbc))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, NULL, pSqlMsg);
    return -1;
  }

  return 0;
}

int dirExec(void *pHEnv, void *pHDbc, char *pSqlStmt, SQLMSG *pSqlMsg)
{
  void *pHStmt = NULL;
  int iRc = 0;

  if(!pHEnv || !pHDbc)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(!pSqlStmt) /* No need to Execute */
    return 0;

  if(SQLAllocStmt(pHDbc, &pHStmt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, NULL, pSqlMsg);
    return -1;
  }

  iRc = SQLExecDirect(pHStmt, pSqlStmt, strlen(pSqlStmt));

  if(iRc != SQL_SUCCESS && iRc != SQL_SUCCESS_WITH_INFO)
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
  }

  if(SQLFreeHandle(SQL_HANDLE_STMT, pHStmt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }

  if(iRc != SQL_SUCCESS && iRc != SQL_SUCCESS_WITH_INFO)
    return -1;

  return 0;
}

int sqlPrep(void *pHEnv, void *pHDbc, char *pSqlStmt, void **pHStmt,
            short *pNumPm, SQLMSG *pSqlMsg)
{
  if(!pHEnv || !pHDbc)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(!pSqlStmt) /* No need to Prepare */
    return 0;

  if(SQLAllocStmt(pHDbc, pHStmt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, NULL, pSqlMsg);
    return -1;
  }

  if(SQLPrepare(*pHStmt, pSqlStmt, SQL_NTS))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, *pHStmt, pSqlMsg);
    return -1;
  }

  if(SQLNumParams(*pHStmt, pNumPm))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, *pHStmt, pSqlMsg);
    return -1;
  }

  return 0;
}

int sqlBind(void *pHEnv, void *pHDbc, void *pHStmt,
            short sNumPm, char *pData, short sPmType, SQLMSG *pSqlMsg)
{
  int iRc = 0;
  short sPmSize = 0;
  char *pTmp = NULL;
  //char tmpBuff[32];

  if(!pHEnv || !pHDbc || !pData)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  switch(sPmType)
  {
    case SQL_CHAR:
    case SQL_VARCHAR:
    case SQL_DATETIME:
                       pTmp = (char *)pData;
                       sPmSize = strlen(pTmp);
                       iRc = SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT,
                                              SQL_C_CHAR, SQL_CHAR, sPmSize,
                                              0, (SQLPOINTER)pTmp, 0, NULL);
                       break;
    case SQL_INTEGER:
#if 0
                       sprintf(tmpBuff, "%ld", *((long *)pData));
                       pTmp = tmpBuff;
                       sPmSize = strlen(pTmp);
#endif
                       iRc = SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT,
                                              SQL_C_LONG, SQL_INTEGER, 4,
                                              0, (SQLPOINTER)pData, 0, NULL);
                       break;
    case SQL_SMALLINT:
#if 0
                       sprintf(tmpBuff, "'%d'", *((short *)pData));
                       pTmp = tmpBuff;
                       sPmSize = strlen(pTmp);
#endif
                       iRc = SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT,
                                              SQL_C_SHORT, SQL_SMALLINT, 2,
                                              0, (SQLPOINTER)pData, 0, NULL);
                       break;
    case SQL_REAL:
#if 0
                       sprintf(tmpBuff, "%f", *((float *)pData));
                       pTmp = tmpBuff;
                       sPmSize = strlen(pTmp);
#endif
                       iRc = SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT,
                                              SQL_C_FLOAT, SQL_REAL, 4,
                                              0, (SQLPOINTER)pData, 0, NULL);
                       break;
    case SQL_DOUBLE:
#if 0
                       sprintf(tmpBuff, "%f", *((double *)pData));
                       pTmp = tmpBuff;
                       sPmSize = strlen(pTmp);
#endif
                       iRc = SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT,
                                              SQL_C_DOUBLE, SQL_DOUBLE, 8,
                                              0, (SQLPOINTER)pData, 0, NULL);
                       break;
  }

  //if(SQLBindParameter(pHStmt, sNumPm, SQL_PARAM_INPUT, SQL_C_CHAR,
                      //SQL_CHAR, sPmSize, 0, (SQLPOINTER)pTmp, 0, NULL))
  if(iRc != SQL_SUCCESS && iRc != SQL_SUCCESS_WITH_INFO)
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }
  
  return 0;
}

int sqlExec(void *pHEnv, void *pHDbc, void *pHStmt,
            int *pRowCnt, short *pNumCols, SQLMSG *pSqlMsg)
{
  if(!pHEnv || !pHDbc || !pHStmt)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(SQLExecute(pHStmt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }

  if(SQLRowCount(pHStmt, (SQLINTEGER FAR *)pRowCnt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }
  
  if(SQLNumResultCols(pHStmt, (short *)pNumCols))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }

  return 0;
}

int sqlDesc(void *pHEnv, void *pHDbc, void *pHStmt,
            short sNumCols, ODBCBind **pDataCols, SQLMSG *pSqlMsg)
{
#if 0
  SQLCHAR     ColName[33];                   /* name of column           */
  SQLSMALLINT ColLen;                        /* length of column name    */
  SQLSMALLINT ColScaling;                    /* scaling of column        */
  SQLSMALLINT ColNullable;                   /* is column nullable?      */
#endif
  SQLSMALLINT tColType;                      /* type of column           */
  SQLUINTEGER tColPrecision;                 /* precision of column      */
  ODBCBind *pTmp = NULL;
  int iRc = 0;
  short i = 0;

  if(!pHEnv || !pHDbc || !pHStmt)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(!(*pDataCols = (ODBCBind *)calloc(sNumCols, sizeof(ODBCBind))))
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  for(i = 0, pTmp = *pDataCols; i < sNumCols; ++i, pTmp++)
  {
#if 0
    if(SQLDescribeCol( hstmt, i+1, ColName, sizeof(ColName) - 1,
                       &ColLen, &ColType, &ColPrecision,
                       &ColScaling, &ColNullable))
#endif
    iRc = SQLDescribeCol(pHStmt, i+1, NULL, 0, NULL, &tColType,
                                        &tColPrecision, NULL, NULL);
    if(iRc != SQL_SUCCESS && iRc != SQL_SUCCESS_WITH_INFO) 
    {
      if(pSqlMsg)
        dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);

      return -1;
    }

    pTmp->tColType = tColType;
    pTmp->tIndicator = 0;

    if(tColPrecision)
    {
      if(!(pTmp->pData = (char *)calloc(1, tColPrecision+1)))
        return -1;
      else
      { 
        if(SQLBindCol(pHStmt, i+1, SQL_C_CHAR, (SQLPOINTER)pTmp->pData,
                      tColPrecision+1, &pTmp->tIndicator))
        {
          if(pSqlMsg)
            dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);

          return -1;
        }
      }
    }
  }

  return 0;
}

int sqlFetch(void *pHEnv, void *pHDbc, void *pHStmt, SQLMSG *pSqlMsg)
{
  int iRc = 0;

  if(!pHEnv || !pHDbc || !pHStmt)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  iRc = SQLFetch(pHStmt);
  if(iRc != SQL_SUCCESS && iRc != SQL_NO_DATA_FOUND &&
     iRc != SQL_SUCCESS_WITH_INFO) 
     
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);

    return -1;
  }
 
  return iRc;
}

int sqlGetOne(void *pHEnv, void *pHDbc, void *pHStmt, const short sType,
              void *pData, const unsigned short usSize,
              const short sClsFlg, SQLMSG *pSqlMsg)
{
  int iRc = 0;

  if(!pHEnv || !pHDbc || !pHStmt || !pData)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(sqlFetch(pHEnv, pHDbc, pHStmt, pSqlMsg) == -1)
    return -1;

  iRc = SQLGetData(pHStmt, 1, sType, pData, usSize, NULL);

  if(iRc != SQL_SUCCESS)
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);

    return -1;
  }

  if(sClsFlg) /* close the cursor */
  {
    if(SQLCloseCursor(pHStmt))
    {
      if(pSqlMsg)
        dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);

      return -1;
    }
  }

  return iRc;
}

int sqlFree(void *pHEnv, void *pHDbc, void *pHStmt,
            short sNumCols, ODBCBind *pDataCols, SQLMSG *pSqlMsg)
{
  ODBCBind *pTmp = NULL;
  short i = 0;

  if(!pHEnv || !pHDbc || !pHStmt)
  {
    if(pSqlMsg)
      pSqlMsg->sqlcd = 0;
    return -1;
  }

  if(pDataCols)
  {
    for(i = 0, pTmp = pDataCols; i < sNumCols; ++i, pTmp++ )
    {
      if(pTmp->pData)
        free(pTmp->pData);
    }

    free(pDataCols);
  }

  if(SQLFreeHandle(SQL_HANDLE_STMT, pHStmt))
  {
    if(pSqlMsg)
      dbErrMsg(pHEnv, pHDbc, pHStmt, pSqlMsg);
    return -1;
  }

  return 0;
}

void dbErrMsg(void *pHEnv, void *pHDbc, void *pHStmt, SQLMSG *pSqlMsg)
{
  static SQLCHAR buffer[SQL_MAX_MESSAGE_LENGTH]; /* message buffer  */
  SQLSMALLINT tLen = 0;                          /* return length  */
  SDWORD tSqlCode = 0;

  SQLError(pHEnv, pHDbc, pHStmt, NULL, &tSqlCode,
           buffer, SQL_MAX_MESSAGE_LENGTH-1, &tLen);

  if(pSqlMsg)
  {
    pSqlMsg->sqlcd = tSqlCode;
    pSqlMsg->pErrMsg = buffer;
  }
}
