/*
 * Copyright (C) 2001 - 2003 Min Hsu
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <qfile.h>
#include <qxml.h>

#include <parsecnfg.h>
#include <printmsg.h>
#include <gmlmonit.h>
 
#ifdef __cplusplus
extern "C" {
#endif

int loadConfig(void *pUser, const char *pConfigFile)
{
  if(pConfigFile)
  {
    ConfigParser tHandler(pUser);
    QFile tXML(pConfigFile);
    QXmlInputSource tSource(tXML);
    QXmlSimpleReader tReader;
    tReader.setContentHandler(&tHandler);
    bool bRc = tReader.parse(tSource);
    tXML.close();
    if(bRc)
      return 0;
    else
      errorMessage("parse XML fail ...\n");
  }

  return -1;
}

inline static SERVERCFG *newConfigItem(void)
{
  SERVERCFG *pChunk = NULL;

  if(!(pChunk = (SERVERCFG *)calloc(1, sizeof(SERVERCFG))))
  {
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 2
    errorMessage("%s\n", strerror(errno));
#else
    errorMessage("calloc failed.\n");
#endif
    return NULL;
  }
  else
    return pChunk;
}

#ifdef __cplusplus
}
#endif

bool ConfigParser::characters(const QString &rChar)
{
  SERVERCFG *pConfig = NULL;

  if(!pMonit)
    return false;

  QString rValue = rChar.simplifyWhiteSpace();

  if(!rValue.isEmpty() )
  {
    if(!(pConfig = newConfigItem()))
      return false;

    pConfig->pParmName = strdup(strTag.ascii());
    if(strType == "char")
      pConfig->pParmCV = strdup(rValue.ascii());
    else if(strType == "int")
    {
      bool bOk = false;

      pConfig->pParmIV = rValue.toInt(&bOk);
      if(!bOk)
        pConfig->pParmIV = 0;
    }

    GDATAPOLL *pPool = newElement(((MONITSRVR *)pMonit)->pSrvrCnfg);
    pPool->pAny = pConfig;
    pPool->pfAnyFree = freeCnfgElement; 
  }

  return true;
}
